#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: chara2PTI.pl,v 1.4 2008/11/12 23:27:51 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;

## chara2PTI -- filter script to convert CHARA input format to 
## canonical PTI format for timing calculations etc. (AB 10/2001)

## Usage: chara2PTI < chara.cat > PTI.cat
##  or blah blah blah | chara2PTI > PTI.cat


## sample PTI GUI format:
## HDC4676 00 48 58.708 +16 56 26.318 -0.002 -0.202 5.1 3.7 F8V... 0.0 xxx   xxx  trg

## sample CHARA format
# ID      102_Vir
# IRC     00240
# 2000.0
# RA      14 19 32.4798
# DEC     -2 15 55.866
# PMRA    -119.290       in time sec?
# PMDEC    -71.870       in arcsec?
# PAR     11.90
# RVEL    -27.1
# MAG      5.15
# BV      1.02
# SP      G8III
# CAT     IRC +00240
# END


## Parse command-line options
use Getopt::Long;
$result = GetOptions("noEcho" => \$noEcho);


#for testing
##open (TEST,"test.cat");

## One.  Slurp input and break it into chunks for each object
$input = "";
while ($line = <STDIN>)
{
    $input .= $line;
}


@objects = split /END/, $input;   # Split the input into object reports



## Two. Process each object systematically

foreach $object (@objects)
{

    @objectLines = split /\n/, $object;

    ## verify this record contains an object designation
    if (grep /ID/, @objectLines)
    {

	## Pull-out and echo comments (as desired)
	@comments = grep /^\#/, @objectLines;
	if (!$noEcho)
	{
	    foreach $comment (@comments)
	    {
		print "$comment\n";
	    }
	}

	## Build the PTI astrometry line

	## First the object designation
	$ptiLine = "";
	@tmp = grep /ID/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 3;
	$ptiLine .= $tmp2[1];


	## Next the object astrometry
	@tmp = grep /RA/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
	$ptiLine .= "   $tmp2[1]";

	@tmp = grep /DEC/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
	$ptiLine .= "   $tmp2[1]";

	@tmp = grep /PMRA/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
        ## TLE - 6/7/2005  must convert pmRA in time sec to arcsec ?
        $tmp2[1] = $tmp2[1]/15.;
	$ptiLine .= "   $tmp2[1]";

	@tmp = grep /PMDEC/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
	$ptiLine .= "   $tmp2[1]";


	## V magnitude (K mag not contained in CHARA format)
	@tmp = grep /MAG/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
	## Write out dummy K magnitude (V).  Note one could guess
	## K mag from V and spectral type using getCal pieces
	$ptiLine .= "   $tmp2[1]  $tmp[1]";


	## B - V
	@tmp = grep /BV/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
	$ptiLine .= " $tmp2[0]";
	

	## Spectral type
	@tmp = grep /SP/, @objectLines;
	@tmp2 = split /\s+/, $tmp[0], 2;
	$ptiLine .= "   $tmp2[1]";

	
	print "$ptiLine\n";
    } ## end if grep /ID/


} ## end foreach $object


exit(0);  ## We're done
