#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: charaFormat.pl,v 1.3 2008/11/12 23:27:51 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;

### charaFormat -- perl script to convert PTI/KI GUI format to CHARA
### format (AB -- 10/2001)

## Usage: charaFormat < in.cat > out.cat
##  or blah blah blah | charaFormat > out.cat

## sample PTI GUI format:
## HDC4676 00 48 58.708 +16 56 26.318 -0.002 -0.202 5.1 3.7 0.51 F8V... 0.0 xxx   xxx  trg

## sample CHARA format
# ID      102_Vir
# IRC     00240
# 2000.0
# RA      14 19 32.4798
# DEC     -2 15 55.866
# PMRA    -119.290
# PMDEC    -71.870
# PAR     11.90
# RVEL    -27.1
# MAG      5.15
# BV      1.02
# SP      G8III
# CAT     IRC +00240
# END


## Parse command-line options
use Getopt::Long;
$result = GetOptions("noEcho" => \$noEcho);


#for testing
##open (TEST,"test.cat");

while ($line = <STDIN>)
{
    if (!($line =~ m/^\#/)
	 && ($line =~ m/\S+/))
    {
	## data line -- parse it
	my ($name,$rah,$ram,$ras,$decd,$decm,$decs,$pmra,$pmdec,
	    $vMag,$kMag,$BminusV,$spType,$rest)
	    = split /\s+/, $line, 14;

	## Mods for sky ideosyncracies -- RLA/AFB
        ## convert units of pmra to seconds-of-time
        $pmra = sprintf("%-6.4f",$pmra/15);

	## print the CHARA-formatted output (note hardwire to J2000 coords)
	print "ID      $name\n";
	print "IRC     xxx\n";
	print "2000.0\n";  ## note J2000 hardwire
	print "RA      $rah $ram $ras\n";
	print "DEC     $decd $decm $decs\n";
	print "PMRA    $pmra\n";
	print "PMDEC   $pmdec\n";
	print "PAR     0.0\n";  ## Note hardwire
	print "RVEL    0.0\n";  ## Note hardwire
	print "MAG     $vMag\n";
	print "BV      $BminusV\n";
	print "SP      $spType\n";
	print "CAT     $name\n";
	print "END\n\n\n";

    }
    elsif(!($noEcho)) ## non-data line -- echo it
    {
	print $line;
    }



} ## end while

print "\n## Processed through charaFormat\n\n";

## end charaFormat
