#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: csAdhoc.pl,v 1.17 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## csAdhoc -- create an adhoc calibration script

our $MODVERSION = '$Id: csAdhoc.pl,v 1.17 2008/11/12 23:28:59 sgroom Exp $';

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use hipRecord;
use hipparcosLine;
use internalLine;
use angleBetween;
use Getopt::Long;
use gcConfig;                       # sigFigures()
use miscArgs;
use astroCatalogue;
use gcSimbad;
use coordinates;

my $cfg = gcConfig::config();       # sigFigures()

## MODIFIED T. Evans   Jun 2005  -- for release 2.8
##   updated to iron out units wrinkles for proper motions and parallaxes:
##     PLX, pmRA and pmDec now will always be in units of arcsec internally

my ($targetDesig, $verbose);

handleCommonOpts(1);
my $result = GetOptions("target=s" => \$targetDesig,
            "sigFigures=i" => sub { $cfg->sigFigures($_[1]); },
			"verbose" => \$verbose
			);

my $catalog = "";
my $targetRecord;
my $targetLine = undef;

if (defined $targetDesig)
{
    print STDERR "DEBUG: csAdHoc: desig $targetDesig\n" if gcConfig::debug();
    my $hipRecordOutput = resolveDesig($targetDesig);
    if (!$hipRecordOutput)
    {
	print STDERR "DEBUG: csAdHoc: hipparcos lookup failed for target $targetDesig\n" if gcConfig::debug();
	print "## error -- no hipparcos record found for $targetDesig\n";
	exit(TARGET_NOT_FOUND);
    }
    $targetLine = new hipparcosLine(line => $hipRecordOutput);
    $catalog .= makeILine($targetDesig,$targetLine,0.0,$cfg->sigFigures());
}


while (my $desig = shift @ARGV)
{

    ## retrieve Hipparcos record for $desig
    my $hipRecordOutput = resolveDesig($desig);
    if (!$hipRecordOutput)
    {
	print "## warning -- no hipparcos record found for $desig\n";
    }
    else
    {
	my $calLine = new hipparcosLine(line => $hipRecordOutput);
	my $sep = &angleBetween($targetLine->extractField("raDeg"),
				$targetLine->extractField("decDeg"),
				$calLine->extractField("raDeg"),
				$calLine->extractField("decDeg"));

	## Append the record to the catalog
	$catalog .= &makeILine($desig,$calLine,$sep,$cfg->sigFigures());
    } ## end defined hipparcos record
} ## end while



## Now the catalog is constructed -- create the calibration script
## using calScript

if ($catalog =~ m/\S+/)
{
    if (defined $targetLine)
    {
	system "echo '$catalog' | calScript -stdin";
    }
    else
    {
	system "echo '$catalog' | calScript -stdin -noTarg";
    }
}

exit(0);  ## and we're done




#########################################################################
## resolveDesig -- resolve the input designation 
#########################################################################
sub resolveDesig
{
## begin resolveDesig  #################################################
    ## resolve the input designation into an hipparcos record
    my $desig = normalIdent(shift @_);
    my @desigPieces = identPieces($desig);
    my ($catalog, $id);
    if ($desigPieces[0] =~ m/HD|HIP/) {
	$catalog = $desigPieces[0];
	$id = $desigPieces[1];
    } else {
	my $targetId = &sansSpaces(&simbadLookupIdent($desig, "HIP|HD"));
	my @pieces = identPieces($targetId);
	$catalog = $pieces[0];
	$id = $pieces[1];
    }
    my $record = undef;
    if ($catalog =~ m/HD|HIP/) {
	$record = getHipRecord($catalog,$id);
    }
    return $record;
} ## end resolveDesig  ##################################################



#########################################################################
## make an internal line representation of a hipparcos line object
#########################################################################
sub makeILine
{
## declarations  ########################################################
    my $desig = shift @_;
    my $catLine = shift @_;
    my $separation = shift @_;
    my $sigFig = shift @_;

## begin ################################################################

    my ($ra,$dec) = $catLine->posJ2000();
    my $hdDesig = join ' ', "HD",$catLine->extractField("HDnum");
    $hdDesig =~ s/\s+/ /g;
    my $hipDesig = join ' ', "HIP",$catLine->extractField("HipID");
    $hipDesig =~ s/\s+/ /g;
    my $intLine
	= new internalLine(designation => $desig,
			   catIndex => $hdDesig,
			   catIndex2 => $hipDesig,
			   tagSPECTYP => $catLine->extractField("tagSPECTYP"),
			   raDeg => $ra,
			   decDeg => $dec,
                           raHMS => &RAhms($ra,$sigFig),
                           decDMS => &DECdms($dec,$sigFig),
			   tagPLX => $catLine->extractField("tagPLX"),
                           pmRA => $catLine->extractField("pmRA"),
			   pmDec => $catLine->extractField("pmDec"),
			   Vmag => $catLine->extractField("Vmag"),
			   BminusV => $catLine->extractField("BminusV"),
			   source => "HIP",
			   tagD => $separation
			   );

    return $intLine->lineOutput();

## end makeILine  #######################################################
}
