#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: fbolFormat.pl,v 1.35 2011/03/23 00:24:37 herstein Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#
# fbolFormat - gather and format photometry for the named object(s)
#
# This version is a rewrite of the "fbolFormat" utility, organized to
# isolate dependencies on SIMBAD, and updated for SIMBAD4 data formats.
# This program is now a simple wrapper for the "formatPhotometry" module.
# The wrapper now only does command line processing, all functionality
# has been moved to the "formatPhotometry" module.
# Steve Groom, February 2007
#
# Object names are taken from the command line after all options have
# have been processed. Photometry is retrieved and formatted for each object.
#
# Photometry is retrieved from several sources:
# - SIMBAD (UBV, uvby, Hbeta, Geneva, JP11, IRC)
# - IRAS
# - 2MASS
# - CIO
# - Hipparcos/Tycho
#
# All measurements for an object are then assembled and output using a
# standard format for further processing by "fbol".

use strict; ## strict syntax checking
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use gcDebug;
use miscArgs;
use miscString;
use formatPhotometry;		# the bulk of the functionality is here

use Carp;

my $cfg = gcConfig::config();

# Step zero.  Handle command-line options

my %opts;

# option defaults
$opts{units} = photometry::UNITS_MAG;

# command line handling
use Getopt::Long;
handleCommonOpts(1);	# do standard getCal opts (-version, -debug, -help, etc)
my $result = GetOptions(
    "fLambda" => sub {$opts{units} = photometry::UNITS_FLAMBDA; },
    "fNu" => sub {$opts{units} = photometry::UNITS_FNU; },
    "noU" => sub {$opts{noU} = 1;},
    "noV" => sub {$opts{noV} = 1;},
    "noB" => sub {$opts{noB} = 1;},
    "noPlx" => sub {$opts{noPlx} = 1;},	#XXX this is not actually implemented?
    "stromgren|uvby" => sub {$opts{uvby} = 1;},
# formatPhotometry is expecting $opts{longWL}, it does not recognize $opts{iras}
#    "longWL|IRAS" => sub {$opts{iras} = 1;},
    "longWL|IRAS" => sub {$opts{longWL} = 1;},
    "2Mass" => sub {$opts{twoMass} = 1;},
    "geneva" => sub {$opts{geneva} = 1;},
    "gezari|CIO" => sub {$opts{gezari} = 1;},
    "hipparcos|tycho" => sub {$opts{hipparcos} = 1;},
    );

my @outLines = formatPhotometry(\@ARGV,undef,\%opts);
print @outLines;

# We're finished.  Exit.
exit(0);
## end fbolFormat  ################################################
