#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: fbolPlot.pl,v 1.8 2008/11/12 23:26:50 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## fbolPlot -- drives fbol plotting using gnuplot


## description -- generates gnuplot script based on fbol input, and
## invokes gnuplot on that script

use strict;  ## use strict syntax checking
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use miscArgs;
use findScript;

my $cfg = gcConfig::config();
## Default values

my $data = "star.dat";
my $model = "bbc.dat";



## Command-line options
my ($help,$longHelp,$version,$designation,$temperature,$tempErr,
    $bolFlux,$bolFluxErr,$angDiam,$angDiamErr,$screen,$ps,$eps,$png);

use Getopt::Long;
handleCommonOpts(1);
Getopt::Long::config("pass_through");
my $result = GetOptions("help" => \$help,
			"longHelp" => \$longHelp,
			"version" => \$version,
			"starName=s" => \$designation,
			"temperature=f" => \$temperature,
			"tempErr=f", => \$tempErr,
			"bolometricFlux=f" => \$bolFlux,
			"bolFluxErr=f" => \$bolFluxErr,
			"angularDiameter=f" => \$angDiam,
			"angDiamErr=f" => \$angDiamErr,
			"screen" => \$screen,
			"ps" => \$ps,
			"PostScript" => \$ps,
			"eps" => \$eps,
			"png" => \$png);


# use designation to create plot title and output filename
my $title = $designation;
$title =~ s/_//g;

my $plotfile = $designation;
$plotfile =~ s/\///g;	# no slash in the filename
$plotfile =~ s/\s+//g;	# no whitespace in the filename
$plotfile =~ s/\.\./-/g;	# no dot-dot in the filename

if (gcConfig::debug())
{
    print STDERR "DEBUG: fbolPlot: designation is $designation\n";
    print STDERR "DEBUG: fbolPlot: title is $title\n";
    print STDERR "DEBUG: fbolPlot: plot filename is $plotfile\n";
}

## Screen plot
if ($screen)
{
    ## Invoke cvscript on the screen template
    ## my $plotTemplate = "./plotTemplate/screenPlot.template";
    my $plotTemplate = findFile("screenPlot.template",$cfg->gcdata);
    die "Error: couldn't find screenPlot.template" if (!(defined $plotTemplate));

    my $cmd = "cvscript $plotTemplate $plotfile $temperature $tempErr $bolFlux $bolFluxErr $angDiam $angDiamErr $title | " . $cfg->gnuplot;
    if (gcConfig::debug())
    {
	print STDERR "DEBUG: fbolPlot: screen: running command \"$cmd\"\n";
    }
    system $cmd;
} ## end doing screen plot



if ($ps)
{

    ## Invoke cvscript on the screen template
    ## my $plotTemplate = "./plotTemplate/psPlot.template";
    my $plotTemplate = findFile("psPlot.template",$cfg->gcdata);
    die "Error: couldn't find psPlot.template" if (!(defined $plotTemplate));

    my $cmd = "cvscript $plotTemplate $plotfile $temperature $tempErr $bolFlux $bolFluxErr $angDiam $angDiamErr $title | " . $cfg->gnuplot;
    if (gcConfig::debug())
    {
	print STDERR "DEBUG: fbolPlot: ps: running command \"$cmd\"\n";
    }
    system $cmd;
} ## end doing PostScript plot



if ($eps)
{
    ## Invoke cvscript on the screen template
    my $plotTemplate = findFile("epsPlot.template", $cfg->gcdata);
    die "Error: couldn't find epsPlot.template" if (!(defined $plotTemplate));
    
    my $cmd = "cvscript $plotTemplate $plotfile $temperature $tempErr $bolFlux $bolFluxErr $angDiam $angDiamErr $title | " . $cfg->gnuplot;
    if (gcConfig::debug())
    {
	print STDERR "DEBUG: fbolPlot: eps: running command \"$cmd\"\n";
    }
    system $cmd;
} ## end doing eps plot



if ($png)
{

    ## Invoke cvscript on the png template
    my $plotTemplate = findFile("pngPlot.template", $cfg->gcdata);
    die "Error: couldn't find pngPlot.template" if (!(defined $plotTemplate));
    
    my $cmd = "cvscript $plotTemplate $plotfile $temperature $tempErr $bolFlux $bolFluxErr $angDiam $angDiamErr $title | " . $cfg->gnuplot;
    if (gcConfig::debug())
    {
	print STDERR "DEBUG: fbolPlot: png: running command \"$cmd\"\n";
    }
    system $cmd;
} ## end doing png plot


exit(0) ## successful completion

