/************************************************************************
*	Copyright (c) 1997-2002 California Institute of Technology      *
*                     U.S. Government Sponsorship                       *
* Under contract with the National Aeronautics and Space Administration *
*************************************************************************/

#ifndef _FLUX_DATA_
#include "Fluxdata.h"
#endif

#ifndef _DATA_LIST_
#include "dataList.h"
#endif

#define INITIAL_VALUE -9999


class StarNode
{
private:
  StarNode *nextNode;

  char *starName;
  int starNum,nTemperatures;
  dataList<Fluxdata> *starData;

  double distance,distanceErr;
  double temperature;

  double BminusV;

  friend class StarList;
};


class StarList
{
private:
  int nodeCount;
  StarNode *head;
  StarNode *current;

public:
  /* Constructor and destructor */
  StarList(void) {nodeCount = 0; head = NULL_NODE;}
  ~StarList(void);

  /* Iterator */
  int iterate(void);
       

  /* Insert/append a node in/to the list */
  void insert(const char *star_name,const int append = 1);
  void insert(const int star_num,const int append = 1);

  /* Pop to the head of the list */
  void gotoHead(void) {current = head;}

  /* Retrieve the data for a particular star */
  int findStar(const char *star_name);
  int findStar(const int nstar);

  /* Retrieve (pointers to) the constituent pieces */
  int numberNodes(void) {return nodeCount;}
  char *getStarName(void) {return current->starName;}
  int getStarNumber(void) {return current->starNum;}

  dataList<Fluxdata> *getStarData(void) {return current->starData;}

  double get_distance(void) {return current->distance;}
  double get_distanceErr(void) {return current->distanceErr;}

  void set_distance(const double dist) {current->distance = (double) dist;}
  void set_distanceErr(const double distErr) {current->distanceErr = (double) distErr;}

  double get_temperature(void) {return current->temperature;}
  void set_temperature(const double temp) {current->temperature = (double) temp;}

  int get_nTemperatures(void) {return current->nTemperatures;}
  void increment_nTemperatures(void) {current->nTemperatures++;}

  double get_BminusV(void) {return current->BminusV;}
  void set_BminusV(const double BmV) {current->BminusV = BmV;}

};

