#include <math.h>
#include <stdlib.h>

#define SWAP(a,b) {double temp=(a);(a)=(b);(b)=temp;}

/* Derived from Numerical Recipes in C, 2nd Edition */
/* Modified to return error code on failure A.B. 7/16/97 */

int gaussJ(double **a,int n,double **b,int m)
{
  int *indxc,*indxr,*ipiv;
  int i,icol,irow,j,k,l,ll,*ivector();
  double big,dum,pivinv;

  indxc = (int *) calloc(n,sizeof(int));
  indxr = (int *) calloc(n,sizeof(int));
  ipiv = (int *) calloc(n,sizeof(int));

  for (j = 0; j < n; j++)
    ipiv[j] = 0; /* Initialize the pivot point */

  for (i = 0; i < n; i++)
    {
      big = 0.0;
      for (j = 0; j < n; j++)
	if (ipiv[j] != 1)
	  for (k = 0; k < n; k++)
	    {
	      if (ipiv[k] == 0)
		{
		  if (fabs(a[j][k]) >= big)
		    {
		      big = fabs(a[j][k]);
		      irow = j;
		      icol = k;
		    } /* end if fabs */
		} /* end ipiv == 0 */
	      else
		if (ipiv[k] > 1)
		  return 1; /* Return singular matrix error condition */
	    } /* end k loop */

      ++(ipiv[icol]);
      if (irow != icol)
	{
	  for (l = 0; l < n; l++)
	    SWAP(a[irow][l],a[icol][l]);
	  for (l = 0;l < m; l++)
	    SWAP(b[irow][l],b[icol][l]);
	} /* end if irow != icol */

      indxr[i] = irow;
      indxc[i] = icol;
      if (a[icol][icol] == 0.0)
	return 1; /* Return singular matrix error condition */
      pivinv = 1.0 / a[icol][icol];
      a[icol][icol] = 1.0;

      for (l = 0; l < n; l++)
	a[icol][l] *= pivinv;

      for (l = 0; l < m; l++)
	b[icol][l] *= pivinv;
	
      for (ll = 0; ll < n; ll++)
	if (ll != icol)
	  {
	    dum = a[ll][icol];
	    a[ll][icol] = 0.0;
	    for (l = 0; l< n; l++)
	      a[ll][l] -= a[icol][l] * dum;
	    for (l = 0;l < m; l++)
	      b[ll][l] -= b[icol][l] * dum;
	  } /* end if/for loop */
    } /* end i loop */

  for (l = n - 1; l >= 0; l--)
    {
      if (indxr[l] != indxc[l])
	for (k = 0; k < n; k++)
	  SWAP(a[k][indxr[l]],a[k][indxc[l]]);
    }

  (void) free(ipiv);
  (void) free(indxr);
  (void) free(indxc);

  return 0; /* nominal return value */

} /* end gaussJ */

#undef SWAP
