#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif


void covSort(double **covar,int ma,int *lista,int mfit)
{
  int i,j;
  double swap;

  for (j = 0; j < ma; j++)
    for (i = j + 1; i < ma; i++)
      covar[i][j] = 0.0;

  for (i = 0; i < mfit; i++)
    for (j = i + 1; j < mfit; j++)
      {
	if (lista[j] > lista[i])
	  covar[lista[j]][lista[i]] = covar[i][j];
	else
	  covar[lista[i]][lista[j]] = covar[i][j];
      } /* end i/j for loops */

  swap = covar[0][0];
  for (j = 0; j < ma; j++)
    {
      covar[0][j] = covar[j][j];
      covar[j][j] = 0.0;
    }

  covar[lista[0]][lista[0]] = swap;
  for (j = 0; j < mfit; j++)
    covar[lista[j]][lista[j]] = covar[0][j];

  for (j = 1; j < ma; j++)
    for (i = 0; i <= j - 1; i++)
      covar[i][j] = covar[j][i];

} /* end covSort */





void mrqCoeff(double *x,double *y,double *sig,int ndata,
	      double *a,int ma,int *lista,int mfit,
	      double **alpha,double *beta,double *chisq,
	      void (*func)(double,double *,double *,double *,int))
{
  int k,j,i;
  double ymod,wt,sig2i,dy;
  /* dyda = dvector(1,ma); */
  double *dyda = (double *) calloc(ma,sizeof(double));


  for (j = 0; j < mfit; j++)
    {
      for (k = 0; k <= j; k++)
	alpha[j][k] = 0.0;
      beta[j] = 0.0;
    }

  *chisq=0.0;

  for (i = 0; i < ndata; i++)
    {
      (*func)(x[i],a,&ymod,dyda,ma);
      sig2i = 1.0 / (sig[i] * sig[i]);
      dy = y[i] - ymod;
      for (j = 0; j < mfit; j++)
	{
	  wt = dyda[lista[j]] * sig2i;
	  for (k = 0; k <= j; k++)
	    alpha[j][k] += wt * dyda[lista[k]];
	  beta[j] += dy * wt;
	} /* end j loop */
      (*chisq) += dy * dy * sig2i;
    } /* end i/data loop */

  for (j = 0; j < mfit; j++)
    for (k = 0; k <= j - 1; k++)
      alpha[k][j] = alpha[j][k];

  (void) free(dyda);

} /* end mrqCoeff */






void mrqMin(double *x,double *y,double *sig,int ndata,
	    double *a,int ma,int *lista,int mfit,
	    double **covar,double **alpha,double *chisq,
	    void (*func)(double,double *,double *,double *,int),
	    double *lambda)

     /* Derived from Numerical Recipes in C, 2nd edition */
{
#define BIG_CHI 1e12

  int k,kk,j,ihit;
  int gjFail = 0;
  static double *da,*atry,**oneda,*beta,ochisq;
  int gaussJ();

  if (*lambda < 0.0)
    {
      /* memory allocation */
      /* oneda = dmatrix(1,mfit,1,1); */
      oneda = (double **) calloc(mfit,sizeof(double *));
      for (k = 0; k < mfit; k++)
	oneda[k] = (double *) calloc(1,sizeof(double));
      
      atry = (double *) calloc(ma,sizeof(double));
      da = (double *) calloc(ma,sizeof(double));
      beta = (double *) calloc(ma,sizeof(double));
      kk = mfit + 1;
      for (j = 0; j < ma; j++)
	{
	  ihit = 0;
	  for (k = 0; k < mfit; k++)
	    if (lista[k] == j)
	      ihit++;

	  if (ihit == 0)
	    lista[kk++] = j;
	  else
	    if (ihit > 1)
	      {
		printf("Bad LISTA permutation in MRQMIN-1\n");
		exit(-1);
	      }
	} /* end j loop */

      if (kk != ma + 1)
	{
	  printf("Bad LISTA permutation in MRQMIN-2\n");
	  exit(-1);
	}

      *lambda = 0.001;
      mrqCoeff(x,y,sig,ndata,a,ma,lista,mfit,alpha,beta,chisq,func);
      ochisq = (*chisq);
    } /* end if lambda < 0 */

  for (j = 0; j < mfit; j++)
    {
      for (k = 0; k < mfit; k++)
	covar[j][k] = alpha[j][k];
      covar[j][j] = alpha[j][j] * (1.0 + (*lambda));
      oneda[j][0] = beta[j];
    }

  /* Modified to trap inversion failures A.B. 7/16/97 */
  if (!(gjFail = gaussJ(covar,mfit,oneda,1)))
    for (j = 0; j < mfit; j++)
      da[j] = oneda[j][0];
  else
    {
      printf("singular covariance matrix in mrqMin\n");
      exit(-1);
    }

  if (*lambda == 0.0)
    {
      covSort(covar,ma,lista,mfit);
      (void) free(beta);
      (void) free(da);
      (void) free(atry);
      /* free_dmatrix(oneda,1,mfit,1,1); */
      return;
    }

  for (j = 0; j < ma; j++)
    atry[j] = a[j];
  for (j = 0; j < mfit; j++)
    atry[lista[j]] = a[lista[j]] + da[j];
  if (!gjFail)
    mrqCoeff(x,y,sig,ndata,atry,ma,lista,mfit,covar,da,chisq,func);
  else
    *chisq = BIG_CHI;

  if (*chisq < ochisq)
    {
      *lambda *= 0.1;
      ochisq=(*chisq);
      for (j = 0; j < mfit; j++)
	{
	  for (k = 0; k < mfit; k++)
	    alpha[j][k] = covar[j][k];
	  beta[j] = da[j];
	  a[lista[j]] = atry[lista[j]];
	}
    }
  else
    {
      *lambda *= 10.0;
      *chisq = ochisq;
    }

  return;

} /* end mrqMin */


