/**************************************************************************/
/* NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu */
/*  Copyright (c) 1997-2008 California Institute of Technology            */
/*                    U.S. Government Sponsorship                         */
/* Under contract with the National Aeronautics and Space Administration  */
/**************************************************************************/
/* Original to Steve Groom, JPL  */
/* 11/20/95                      */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif

#if HAVE_STDIO_H
#include <stdio.h>
#endif

#if HAVE_UNISTD_H
#include <unistd.h>
#endif

#if HAVE_FCNTL_H
#include <fcntl.h>
#endif

#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

// Spawn the program named by "file" with argument list argv[] as a separate
// process, using the user's search path.  Stdin and stdout of the new process
// are connected to the current process via pipes. Pipe_fd is a pointer
// to a two-integer array, in which will be returned file descriptors which
// are pipes to the new process' stdin/stdout. (pipe_fd[0] will be data
// from the new process' stdout, pipe_fd[1] will be data to the new
// process' stdin.)  If pipe_fd is given as NULL, then no pipes
// are established, and the child's stdin/stdout are redirected to /dev/null.
// In both cases the child's stderr is not redirected, and remains shared
// with the parent.
// Returns pid of child on success, or prints message and returns 0 on error.

pid_t spawnChild(char *file, char *argv[], int pipe_fd[])
{

    int pipe_out[2];
    int pipe_in[2];
    pid_t pid;

    if (pipe_fd != NULL)
    {
	if ((pipe(pipe_in) == -1) || (pipe(pipe_out) == -1))
	{
	    perror("spawnChild: pipe");
	    fprintf(stderr,"spawnChild: can't create pipe to spawn %s\n",file);
	    return (pid_t) 0;
	} // end pipe create failure
    } // end pipe pointers desired

    pid = fork();

    if (pid == (pid_t) -1)			// error
    {
	perror("spawnChild: fork");
	fprintf(stderr,"spawnChild: can't fork to spawn %s\n",file);
	if (pipe_fd != NULL)
	{
	    (void) close(pipe_out[0]);
	    (void) close(pipe_out[1]);
	    (void) close(pipe_in[0]);
	    (void) close(pipe_in[1]);
	}
	return (pid_t) 0;
    }

    if (pid != (pid_t) 0)			// parent
    {
	if (pipe_fd != NULL)
	{
	    // setup pipe to child
	    (void) close(pipe_out[0]);
	    (void) close(pipe_in[1]);
	    pipe_fd[0] = pipe_in[0];	// in from child's stdout
	    pipe_fd[1] = pipe_out[1];	// out to child's stdin
	}
	return pid;		
    }
    else
    {					// child
	// setup stdin/stdout
	if (pipe_fd == NULL)
	{
	    // send stdin/stdout to /dev/null
	    int fd0,fd1;
	    if (((fd0=open("/dev/null",O_RDONLY)) == -1) || 
	        ((fd1=open("/dev/null",O_WRONLY)) == -1))
	    {
		perror("spawnChild: execvp");
		fprintf(stderr,"spawnChild: error redirecting I/O\n");
		exit(1);
	    }
	    else
	    {
		dup2(fd0,0);		// child's stdin to /dev/null
		dup2(fd1,1);		// child's stdout to /dev/null
		(void) close(fd0);
		(void) close(fd1);
	    }
	}
	else
	{
	    // setup stdin/stdout into pipe
	    dup2(pipe_out[0],0);	// child's stdin
	    dup2(pipe_in[1],1);		// child's stdout
	    (void) close(pipe_in[0]);
	    (void) close(pipe_in[1]);
	    (void) close(pipe_out[0]);
	    (void) close(pipe_out[1]);
	}
	// exec program
	if (execvp(file,argv) == -1)
	{
	    perror("spawnChild: execvp");
	    fprintf(stderr,"spawnChild: error on exec %s\n",file);
	    exit(1);
	}
	/*NOTREACHED*/
    }
}


