#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: gcConf.pl,v 1.17 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#
# gcConf - dump getCal's idea of the user's runtime environment
#
# The runtime environment is a combination of:
# 	- hardcoded default values
# 	- build-time settings
# 	- settings from the getCal configuration file
# 	- settings from environment variables

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
my $bindir = "$RealBin";			# location of getCal Perl programs
my $libdir = "$RealBin/../lib/modules";	# location of getCal Perl modules

use gcConfig;
use stringSets;

sub printPaths
{
    my $title = shift;
    my $paths = shift;
    if ($paths and scalar(@$paths)) {
	print "$title:\n";
	for my $p (@$paths) {
	    print "\t$p\n";
	}
	print "\n";
    }
}

sub showPaths
{
    my $entity = shift;
    my $configval = shift;
    my $varname = shift;
    my $title = "Search path(s) for $entity ";
    my @paths;
    if (defined($ENV{$varname})) {
	$title .= "[from environment variable $varname; this overrides the value of $configval as set in the build-time configure script]";
	@paths = split(":", $ENV{$varname});
    } else {
	$title .= "[from the build-time configure script]";
	@paths = ($configval, );
    }
    printPaths($title, \@paths);
}

print "############################################\n";
print "Current configuration information for getCal\n";
print "############################################\n";
print "\n";
printf "getCal version                            %s\n", gcConfig::version_full();
print "\n";

printf "Current bin directory                     %s\n", $bindir;
printf "Default location of getCal Perl modules   %s\n", $libdir;
printf "\n";

my $cfile = gcConfig::gcConfigFile();
printf "Current configuration file                %s\n", $cfile;
if (! -f $cfile) {
    print "Note: this configuration file currently does not exist.\n";
    my $example = $cfile . ".example";
    if (-f $example) {
	print "A sample configuration file does exist at\n$example,\nand you can copy it to $cfile.\n";
    }
}
print "Configuration file location can be overridden by setting the GC_CONFIG\n";
print "environment variable. Run-time parameters in the configuration file\n";
print "can be overridden using environment variables.\n\n";

print "Build-time configuration:\n";
printf "Build directory                           %s\n", gcConfig::build_path();
printf "Installation prefix                       %s\n", gcConfig::prefix();
printf "Installation bin directory                %s\n", gcConfig::bindir();
print "\n";

# Display the Perl module search paths, keeping in mind that PERLLIB is not
# used if PERL5LIB is set.
my @perl_paths;
if (defined($ENV{PERL5LIB}))
{
    @perl_paths = split(':', $ENV{PERL5LIB});
    printPaths("Search path(s) for Perl modules (from environment variable PERL5LIB)", \@perl_paths);
}
elsif (defined($ENV{PERLLIB}))
{
    @perl_paths = split(':', $ENV{PERLLIB});
    printPaths("Search path(s) for Perl modules (from environment variable PERLLIB)", \@perl_paths);
}
my $other_paths = diff(\@INC, \@perl_paths);
printPaths("Search path(s) for Perl modules (from the site configuration)", $other_paths);

print "\n";

print "Parameters in effect:\n";
my $cfg = gcConfig::config(1);
exit(0);
