#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: gcList.pl,v 1.10 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use miscArgs;
use Getopt::Long;

## gcList -- wrapper script for getCal that allows the getCal
## processing of a list.  A.B. 9/30/99


## Typical usage: gcList -objDesig [name|HD|HIP] [getCal arguments] < objectList

## Alternate usage: cat | gcList -objDesig [name|HD|HIP] [getCal arguments] 
## obj1
## obj2
## cntrl-d

handleCommonOpts(1, "gcList.txt");
my $desig;
my $result = GetOptions("objDesig=s" => \$desig);

my $gcArgStr = join ' ', @ARGV;
my $objSpec;
if ($desig && $desig =~ m/HD/)
{
    $objSpec = "-targetHD";
}
elsif ($desig && $desig =~ m/HIP/)
{
    $objSpec = "-targetHIP";
}
else
{
    $objSpec = "-targetName";
}

while (my $obj = <STDIN>)
{
    if (($obj =~ m/\S+/)      # Don't process trivial lines or
	&& !($obj =~ m/^\#/)) #  comment lines
    {
	## Object designation transformations
	chomp($obj); ## lose the newline
	$obj =~ s/\#.+//g;  ## eliminate trailing comments
	$obj =~ s/\s+\z//;  ## eliminate any trailing spaces
	$obj =~ s/\s+/_/g;  ## substitute underlines for remaining spaces

	if ($obj =~ m/\S+/) ## If the object is STILL non-trivial
	{
	   my $arg = join ' ', $objSpec,$obj,$gcArgStr;  # compose the actual argument to getCal
	   print "\n\n## executing getCal $arg\n\n";
	   my $gcOut = `getCal $arg` or die;
	   print $gcOut;
        }

    } ### end non-trivial line check


} ### end while loop


### end gcList
