package guiSubs;
my $MODVERSION = '$Id: guiSubs.pm,v 1.15 2008/11/12 23:28:13 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

# This module contains subroutines that were previously found copy-pasted
# into multiple GUI files.  Yuck!

use strict;
use warnings;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(xephemOutput simbadName doSimbadGui dispText editTimingArgument getOutputName writePsOutput);


use gcConfig;
use astroCatalogue;
use miscString;

my $cfg = config();

#########################################################################
## xephemOutput - output the current object catalog to xephem
#########################################################################
sub xephemOutput
{
## declarations xephemOutput  ###########################################
    my $MW = shift @_;
    my $catalog = shift @_;
    my $fifoMode = shift @_;

    ## Initialize default file selection
    my $initDir = $ENV{PWD};
    my $initFile = "xephemCatalog.edb";

## begin xephemOutput  ##################################################

    ## Push the current object catalog through xeConvert
    my $xeCat = `echo "$catalog" | xeConvert -stdin` or die;

    ## Look for the Xephem fifo
    if ((defined $fifoMode)
	&& ($fifoMode))
    {
	$initDir = $cfg->xephemFifo;
	my @tmp = split /\//, $initDir;
	$initFile = $tmp[-1];
        $initDir =~ s/$initFile//;
    }

    ## Filename user interaction
    my $fn = $MW->getSaveFile(-initialdir => $initDir,
			      -initialfile => $initFile,
			      -defaultextension => ".edb",
			      -filetypes => [ ['XEphem catalog files', '.edb'],
					      ['XEphem FIFO files', '' ],
					      ['All Files' , '*'] ],
			      -title => "Save XEphem catalog file");
    ## Do the file output
    if ($fn)
    {

	## Open in create/write-only/append/non-blocking mode
	##  (this last in case this is the XEphem FIFO)
	if ((-w $fn) && (-p $fn)) ## writable fifo test
	{
	    open(CAT,"+>>$fn") or die "failed opening fifo target $fn\n";
        }
	else  ## not fifo (regular catalog)
	{
	    open(CAT,">>$fn") or die "failed opening catalog target $fn\n";
	}
	
	print CAT "$xeCat\n\n";
	close(CAT);
	print "XEphem catalog appended to $fn\n";
    }  ## end good filename

} ## end xephemOutput  ##################################################


#########################################################################
## simbadName -- name manipulations for simbad
#########################################################################
sub simbadName
{
    ## Make our names more palatable for Simbad's consumption
    ## leaving whitespace as underscores. (simbadInteractiveQuery will fix that)
    my $arg = $_[0];
    my $ident = normalIdent($arg);
    $ident =~ s/\s+/_/g;	## replace whitespace
    print STDERR "DEBUG: simbadName: \"$arg\" changed to \"$ident\"\n" if gcConfig::debug();
    return $ident;
} ## end simbadName  ####################################################

#########################################################################
## doSimbadGui -- do a simbadInteractiveQuery on a list of targets and
##     display the output in a GUI window
#########################################################################
sub doSimbadGui
{
    my $MW = shift;	# the main window
    my @tlist = @_;	# list of target names

## begin doSimbadGui  ######################################################
    ## SG 4/6/06 - need a better test for whether the user wants to use the
    ## browser or not. In 2.8, this test is always true, so disabling the
    ## browser option here until we come up with a better test.
    ##
    ## if ($cfg->browser)
    ## {
    ## 	system "simbadInteractiveQuery -browser @tlist";
    ## }
    ## else
    ## {
	my $cmd = "simbadInteractiveQuery @tlist";
	print STDERR "DEBUG: doSimbadGui: running command \"$cmd\"\n" if gcConfig::debug();
	my $result = `$cmd`;
	&dispText($MW,$result);
    ## }
} ## end doSimbadGui  ####################################################

#########################################################################
## dispText -- display a read-only text window with generic text, with color highlighting
#########################################################################
sub dispText
{
    my $MW = shift;			# the main window
    my $textInput = shift;		# the text to display, may contain newlines
    my $title = shift;			# optional window title
    my $blueKeys = shift;		# optional list of keys to highlight in blue

    ## Color highlighting scheme:
    ## - The (optional) supplied string $blueKeys contains a pattern, any lines that match
    ##   are displayed in blue.
    ## - Any comment lines (# at beginning of line) are displayed in black
    ## - Any non-comment lines are displayed in red

## begin dispText  ######################################################
    ## Basically we open a text window and display the input text -- in
    ## all it's glory.

    if (!$MW)
    {
	print STDERR "DEBUG: dispText: no main window supplied\n" if gcConfig::debug();
	return;
    }

    ## create a text window to display the return info
    my $helpWindow = $MW->Toplevel();
    if ($title)
    {
	$helpWindow->title("text display -- $title");
    }
    else
    {
	$helpWindow->title("text display");
    }

    my $text = $helpWindow->Scrolled("ROText",
				     -wrap => "none",
				     -width => 90, -height => 32,
				     -relief => "groove"
				     )->pack(-side => "top",
					 -expand => "yes",
					 -fill => "both",
				     );

    # dump the argument text into the text window
    # Add the text with some color-coding to help the reader
    $text->tagConfigure('colorBlue',-foreground => "blue");
    $text->tagConfigure('colorRed',-foreground => "red");
    if ($textInput)
    {
	my @tmpTxt = split /\n/, $textInput;
	foreach my $line (@tmpTxt)
	{
	    $line .= "\n";
	    if (($blueKeys) && ($line =~ m/$blueKeys/))
	    {
		# selected lines in blue
		$text->insert('end',$line,'colorBlue');
	    }
	    elsif ($line =~ m/^\#/)
	    {
		# additional comment lines in black
		$text->insert('end',$line);
	    }
	    else
	    {
		# non-comment lines in red
		$text->insert('end',$line,'colorRed');
	    }
	} # end foreach $line
    }


    # add save and close buttons
    my $hwCntrl = $helpWindow->Frame(-relief => "groove",
				     -borderwidth => "2"
				     )->pack(-side => "bottom", -fill => "x");
    $hwCntrl->Button(-text => "Save",
		     -relief => "raised",
		     -command => sub
		     {
			 my $file = $helpWindow->getSaveFile;
			 
			 if ($file)
			 {
			     open (OUTPUT, ">>$file") or print "Couldnt open $file";
			     print OUTPUT "$textInput\n\n";
			     close OUTPUT;
			     print "timing information appended to $file\n";
			 }  ## end if file
		     }
		     )->pack(-side => "left");

    $hwCntrl->Button(-text => "Close",
		     -relief => "raised",
		     -command => sub {$helpWindow->withdraw}
		     )->pack(-side => "right");

    $helpWindow->bind('<Return>' => sub { $helpWindow->withdraw });

} ## end dispText  ######################################################


#########################################################################
## editTimingArgument -- facility for controling timing recomputation
#########################################################################
sub editTimingArgument
{
    ## editTimingArgument opens up a simple window to get the timing
    ## arguments from the user
## declarations editTimingArgument  #####################################
    my $argument = shift @_;

## begin editTimingArgument  ############################################
    my $editTArg = shift @_;

    $editTArg->title("timing argument input");


    my $tArgFrame = $editTArg->Frame(-relief => "raised",
				     -borderwidth => "2"
				     )->pack(-side => "top", -fill => "x");

    $tArgFrame->Label(-text => "timing arguments: ",
		      )->pack(-side => "left", -anchor => "w");
    
    $tArgFrame->Entry(-textvariable => \$argument,
		      -relief => 'raised', -width => 80
		      )->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);

    my $buttonFrame = $editTArg->Frame(-relief => "raised",
				       -borderwidth => "2"
				       )->pack(-side => "top", -fill => "x");

    $buttonFrame->Button(-text => "Go",
			 -relief => 'groove',
			 -borderwidth => "5",
			 -command => sub 
			 {
			     $editTArg->destroy();
			 }
			 )->pack(-side => "left");

    $buttonFrame->Button(-text => "timing Help",
			 -relief => 'groove',
			 -borderwidth => "5",
			 -command => sub 
			 {
			     &dispHelp(findFile("timing.txt",$cfg->gcdocs, $cfg->gcdata));
			 }
			 )->pack(-side => "left");


    my $cancel = 0;  ## Give the option to cancel re-timing
    $buttonFrame->Button(-text => "Cancel",
			 -relief => 'groove',
			 -borderwidth => "5",
			 -command => sub 
			 {
			     $cancel = 1;
			     $editTArg->destroy();
			 }
			 )->pack(-side => "right");

    $editTArg->bind('<Return>' => sub  ## bind Return to go
		    {
			$editTArg->destroy();
		    });

    $editTArg->waitWindow();

    return ($argument,$cancel);

} ## end editTimingArgument  ############################################


#########################################################################
## getOutputName -- get output information from user
#########################################################################
sub getOutputName
{
    ## getOutputName opens up a simple window to get a printer info from user
    ## output can be either a print command, or a filename to save the output
    my $pickFile = shift @_; #$MW->Toplevel();
    my $balloon = shift @_;
    my $fileName = shift @_;

## declarations getOutputName  ############################################
    my $printCommand = "lpr";


    $pickFile->title("Filename Selection");

## begin getOutputName  ###################################################

    ## Printing to printer
    my $printFrame = $pickFile->Frame(-relief => "raised",
				      -borderwidth => "2"
				      )->pack(-side => "top", -fill => "x");


    $printFrame->Label(-text => "Print Command",
		       )->pack(-side => "left", -anchor => "w");
    
    my $e1 = 
	$printFrame->Entry(-textvariable => \$printCommand,
			   -relief => 'raised'
			   )->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);
    $balloon->attach($e1,-msg => "Print command entry");

    my $printer = 1;
    my $printButton = $printFrame->Button(-text => "Send to Printer",
					  -relief => 'groove',
					  -borderwidth => "5",
					  -command => sub 
					  {
					      if ($fileName)
					      {
						  $pickFile->destroy();
					      }
					  }
					  )->pack(-side => "left");
    $balloon->attach($printButton,-msg => "Send timing graphic to printer using print command");


    ## Saving to a file
    my $saveFrame = $pickFile->Frame(-relief => "raised",
				     -borderwidth => "2"
				     )->pack(-side => "top", -fill => "x");

    $saveFrame->Label(-text => "Filename",
		      )->pack(-side => "left", -anchor => "w");
    
    my $e2 =
	$saveFrame->Entry(-textvariable => \$fileName,
			  -relief => 'raised'
			  )->pack(-side => "left", -anchor => "w", -fill => "x", -expand => 1);
    $balloon->attach($e2,-msg => "Filename for save");

    ## Radio (mutually exclusive) buttons to define the type of
    ## output we're producing.  Default to PostScript
    my $fileType = "ps";
    $fileName =~ s/\.ps|\.pdf|\.gif|\.jpg|\.png//g;
    $fileName .= "." . $fileType;
    my $objRadio;
    foreach (qw(ps pdf gif jpg png))
    {
	$objRadio = $saveFrame->Radiobutton(-text => $_,
					    -value => $_,
					    -variable => \$fileType,
					    -command => sub
					    {
						# set suffix to match type
						$fileName =~ s/\.ps|\.pdf|\.gif|\.jpg|\.png//g;
						$fileName .= "." . $fileType;
					    }
					    )->pack(-side => "left");
    } # end foreach

    my $fileButton = $saveFrame->Button(-text => "Save To Filename",
					-relief => 'groove',
					-borderwidth => "5",
					-command => sub 
					{
					    $printer = 0;
					    if ($fileName)
					    {
						$pickFile->destroy();
					    }
					}
					)->pack(-side => "left");
    $balloon->attach($fileButton,-msg => "Save timing graphic to filename");

    ## Cancel -- forget about it.
    my $cancelFrame = $pickFile->Frame(-relief => "raised",
				       -borderwidth => "2"
				       )->pack(-side => "top", -fill => "x");
    my $cancel = 0;
    my $b1 =
	$cancelFrame->Button(-text => "Cancel",
			     -relief => 'raised',
			     -command => sub 
			     {
				 $printer = 0;
				 $cancel = 1;
				 $pickFile->destroy();
			     }
			     )->pack(-side => "right");
    $balloon->attach($b1,-msg => "Cancel printing/file save");

    $pickFile->bind('<Return>' => sub
		    {
			$printButton->select();
			$pickFile->destroy();
		    });

    $pickFile->waitWindow();

    if ($cancel)
    {
	return ""; ## return empty filename
    }
    if ($printer && $printCommand)
    {
	return join '',"|",$printCommand; ## Returning a print command
    }
    elsif (!$printer && $fileName)
    {
	return $fileName;	## output to file with suffix indicating the type
    } ## end saving to file
    return "";
} ## end getOutputName  ###################################################


#########################################################################
## writePsOutput -- write postscript diagram to given file or command
#########################################################################
sub writePsOutput
{
## writePsOutput writes the given PostScript text to the named file or
## command, performing file format conversions implied by the filename.
##
## -- if the filename starts with "|", then it is treated as a command,
##    and the postscript is piped to that command with no format conversion.
## 
## -- otherwise, the following suffixes are recognized, with the postscript
##    converted to the desired format:
##    .ps - write postscript output directly to file
##    .gif - convert postscript to GIF file
##    .jpg - convert postscript to JPEG file
##    .png - convert postscript to PNG file
##    .pdf - convert postscript to PDF file
##
## declarations writePsOutput  ###########################################
    my $fn = shift @_; # filename (or if it starts with '|', command)
    my $ps = shift @_; # postscript text to be written

    # location of various utilities we may need
    my $pstopnm = $cfg->pstopnm;
    my $pnmflip = $cfg->pnmflip;
    my $ppmtogif = $cfg->ppmtogif;
    my $ppmtojpeg = $cfg->ppmtojpeg;
    my $pnmtopng = $cfg->pnmtopng;
    my $ps2pdf = $cfg->ps2pdf;

## begin writePsOutput  ##################################################
    if ($fn && ($fn =~ m/^\|/))
    {
	# pipe output to command
	my $cmd = $fn;
	$cmd =~ s/^\|//;	# remove the pipe symbol, exec_stdin handles that
	print STDERR "DEBUG: writePsOutput: printing with command $cmd\n" if gcConfig::debug();
	exec_stdin($cmd, $ps, gcConfig::debug());
    }
    elsif ($fn)
    {
	# Write output to file, converting postscript to other formats as requested

	if ($fn =~ m/.ps$/) ### Write PostScript output to file###
	{
	    print STDERR "DEBUG: writePsOutput: writing to postscript file $fn\n" if gcConfig::debug();
	    open (PS,">",$fn) or die "Error opening $fn\n";
	    print PS $ps;
	    close (PS);
	    print "Wrote PostScript file $fn\n";
	}

	if ($fn =~ m/.gif$/) ### Make gif image as requested ###
	{
	    if (!$pstopnm || !$pnmflip || !$ppmtogif)
	    {
		print STDERR "GIF output not available - required utilities not found\n";
	    }
	    else
	    {
		my $cmd = "$pstopnm -stdout -xborder 0 -yborder 0 -landscape -dpi 100 - | $pnmflip -r180 | $ppmtogif > $fn";
		print STDERR "DEBUG: writePsOutput: running conversion command: $cmd\n" if gcConfig::debug();
		exec_stdin($cmd, $ps, gcConfig::debug());
		print "Wrote GIF file $fn\n";
	    }
	} ## end gif writing

	if ($fn =~ m/.jpg$/) ### Make jpg image as requested ###
	{
	    if (!$pstopnm || !$pnmflip || !$ppmtojpeg)
	    {
		print STDERR "JPEG output not available - required utilities not found\n";
	    }
	    else
	    {
		my $cmd = "$pstopnm -stdout -xborder 0 -yborder 0 -landscape -dpi 100 - | $pnmflip -r180 | $ppmtojpeg > $fn";
		print STDERR "DEBUG: writePsOutput: running conversion command: $cmd\n" if gcConfig::debug();
		exec_stdin($cmd, $ps, gcConfig::debug());
		print "Wrote JPEG file $fn\n";
	    }
	} ## end jpeg writing

	if ($fn =~ m/.png$/) ### Make png image as requested ###
	{
	    if (!$pstopnm || !$pnmflip || !$pnmtopng)
	    {
		print STDERR "PNG output not available - required utilities not found\n";
	    }
	    else
	    {
		my $cmd = "$pstopnm -stdout -xborder 0 -yborder 0 -landscape -dpi 100 - | $pnmflip -r180 | $pnmtopng > $fn";
		print STDERR "DEBUG: writePsOutput: running conversion command: $cmd\n" if gcConfig::debug();
		exec_stdin($cmd, $ps, gcConfig::debug());
		print "Wrote PNG file $fn\n";
	    }
	} ## end png writing

	if ($fn =~ m/.pdf$/) ### Make pdf image as requested ###
	{
	    if (!$ps2pdf)
	    {
		print STDERR "PDF output not available - required utilities not found\n";
	    }
	    else
	    {
		my $cmd = "$ps2pdf -sOutputFile=$fn -";
		print STDERR "DEBUG: writePsOutput: running conversion command: $cmd\n" if gcConfig::debug();
		exec_stdin($cmd, $ps, gcConfig::debug());
		print "Wrote PDF file $fn\n";
	    }
	} ## end pdf writing
    }
} ## end writePsOutput  ##################################################


# Candidates for centralization
# grep -rE "^sub " * | awk -F':' '{print $2":"$1}' | sort |less

# packSimbadMenu
# dispHelp
# packTargetMenu
# readTimingRpt
# relativeTimings
# xCoord
# drawAxes / drawAxis
# drawCurrentUT
# drawPreamble
# drawTargetLine
# drawTargets

# setCurrentTime
# packBlineMenu
