package findScript;
my $MODVERSION = '$Id: findScript.pm,v 1.8 2008/11/12 23:28:22 sgroom Exp $';
require 5.000;

use strict;
use warnings;
use gcConfig;

use Exporter ();
our ($VERSION, @ISA, @EXPORT);
$VERSION=1.0;
@ISA = qw(Exporter);
@EXPORT = qw(findScript findFile);

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## findScript routines find the appropriate file or executable for use
## in the autotools version of getCal


#########################################################################
## findScript -- find executable script or program
#########################################################################
sub findScript
{
## begin findScript  ####################################################
    my $target = shift @_;  ## the script target
    my @pathList = @_; ## the search path list
    foreach my $path (@pathList)
    {
	my $trg = join '/', $path,$target;

	if (-x $trg) ## return first executable instance of $target
	{
	    return $trg;
	}

    } ## end foreach

    ## if we're here it means we didn't find $target -- return
    ## undefined status

    return undef;

} ## end findScript  ####################################################




#########################################################################
## findFile -- find a generic file (e.g. documentation)
#########################################################################
sub findFile
{
## begin findFile  ######################################################
    my $target = shift @_;  ## the file target
    my @pathList = @_; ## the search path list
    foreach my $path (@pathList)
    {
	my $trg = join '/', $path,$target;
	if (-f $trg) ## return first readable instance of $target
	{
	    return $trg;
	}

    } ## end foreach

    ## if we're here it means we didn't find $target -- return
    ## undefined status

    return undef;

} ## end findFile  ######################################################


return 1; ## end findScript.pm
