package formatConv;
my $MODVERSION = '$Id: formatConv.pm,v 1.7 2006/12/21 20:32:45 sgroom Exp $';

use strict;
use warnings;
use internalLine;
use coordinates;
use gcSimbad;
use miscString;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw( hip2ILine simbad2ILine );


#########################################################################
## make an internal line object, given a Hipparcos line object
## 
#########################################################################
sub hip2ILine
{
    my $hipLine = shift @_;

    my ($ra,$dec) = $hipLine->posJ2000();

    my $hdDesig = join ' ', "HD",$hipLine->extractField("HDnum");
    if ($hdDesig =~ m/\d+/)
    {
	$hdDesig =~ s/\s+/ /g;
    }
    else
    {
	$hdDesig = "";
    }
    my $hipDesig = join ' ', "HIP",$hipLine->extractField("HipID");
    if ($hipDesig =~ m/\d+/)
    {
	$hipDesig =~ s/\s+/ /g;
    }
    else
    {
	$hipDesig = "";
    }

    my $intLine = new internalLine (catIndex => $hdDesig,
			   catIndex2 => $hipDesig,
			   tagSPECTYP => $hipLine->extractField("spType"),
			   raDeg => $ra,
			   decDeg => $dec,
			   tagPLX => $hipLine->extractField("parallax"),
			   pmRA => $hipLine->extractField("pmRA"),
			   pmDec => $hipLine->extractField("pmDec"),
			   Vmag => $hipLine->extractField("Vmag"),
			   BminusV => $hipLine->extractField("BminusV"),
			   source => "HIP"
			   );
    return $intLine;
}

# Set parameters in an InternalLine given a Simbad result object;
# Don't overwrite any data contained in the optional iLine argument.
sub simbad2ILine {
    my $simbadObj = shift;
    my $iLine = shift;
     
    $iLine = new internalLine() if (!$iLine);
    $iLine->setField("raDeg", $simbadObj->raDeg()) if (!$iLine->{raDeg});
    $iLine->setField("decDeg", $simbadObj->decDeg()) if (!$iLine->{decDeg});
    $iLine->setField("tagSPECTYP", $simbadObj->spec()) if (!$iLine->{tagSPECTYP});
    $iLine->setField("Vmag", $simbadObj->vmag()) if (!$iLine->{Vmag});
    $iLine->setField("BminusV", $simbadObj->bmag() - $simbadObj->vmag()) if (!$iLine->{BminusV} && isNumeric($simbadObj->bmag()) && isNumeric($simbadObj->vmag()));
    $iLine->setField("tagPLX", $simbadObj->plx()) if (!$iLine->{tagPLX});

    # Proper motion from Simbad in arcsec/year
    $iLine->setField("pmRA", $simbadObj->pmra()) if (!$iLine->{pmRA});
    $iLine->setField("pmDec", $simbadObj->pmdec()) if (!$iLine->{pmDec});
    return $iLine;
}
