package hipparcosLine;
my $MODVERSION = '$Id: hipparcosLine.pm,v 1.8 2008/11/12 23:28:23 sgroom Exp $';
require 5.000;
use strict;
use warnings;

##use Exporter();
##
##our @ISA = qw(Exporter);
##our @EXPORT = qw(
##	     );


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

## hipparcosLine.pm -- code/class library for dealing with the
## hipparcos catalog format


use angles;  ## angle constants
use miscString;
use coordinates;



#########################################################################
## global declarations
#########################################################################
my $fieldDelimiter = "|";  ## Hipparcos catalog field delimiter


# Access index keys for the Hipparcos catalog fields

my $HIP_Num = 1;
my $HIP_ProxFlag = 2;
my $HIP_RAhms = 3;
my $HIP_DECdms = 4;
my $HIP_Vmag = 5;
my $HIP_VarFlag = 6;
my $HIP_sourceV = 7;
my $HIP_RAdeg = 8;
my $HIP_DECdeg = 9;
my $HIP_Plx = 11;
my $HIP_pmRA = 12;
my $HIP_pmDEC = 13;
my $HIP_eRA = 14;
my $HIP_eDEC = 15;
my $HIP_ePlx = 16;
my $HIP_epmRA = 17;
my $HIP_epmDEC = 18;
my $HIP_BTycho = 32;
my $HIP_BTyErr = 33;
my $HIP_VTycho = 34;
my $HIP_VmagErr = 35;
my $HIP_BminusV = 37;
my $HIP_BmVErr = 38;
my $HIP_VminusI = 40;
my $HIP_VmIErr = 41;
my $HIP_Hpmag = 44;
my $HIP_e_Hpmag = 45;
my $HIP_Hpscat = 46;
my $HIP_o_Hpmag = 47;
my $HIP_DSFlag = 59;
my $HIP_AstSrcFlag = 60;
my $HIP_SolQual = 61;
my $HIP_PosAng = 63;
my $HIP_rho = 64;
my $HIP_e_rho = 65;
my $HIP_deltaH = 66;
my $HIP_HDnum = 71;
my $HIP_DMname = 72;
my $HIP_DMname2 = 73;
my $HIP_spType = 76;


## Additional fields we add as we process
my $HIP_AppAngDist = 78;
my $HIP_AppKmag = 79;
my $HIP_AppEAS = 80;
my $HIP_AppEASErr = 81;

## Big hash mapping names to fields
my %HipparcosKeys =
    (
     ## older-style getCal designations
     "HIP_Num" => $HIP_Num,
     "HIP_ProxFlag" => $HIP_ProxFlag,
     "HIP_RAhms" => $HIP_RAhms,
     "HIP_DECdms" => $HIP_DECdms,
     "HIP_Vmag" => $HIP_Vmag,
     "HIP_VarFlag" => $HIP_VarFlag,
     "HIP_RAdeg" => $HIP_RAdeg,
     "HIP_DECdeg" => $HIP_DECdeg,
     "HIP_Plx" => $HIP_Plx,
     "HIP_pmRA" => $HIP_pmRA,
     "HIP_pmDEC" => $HIP_pmDEC,
     "HIP_eRA" => $HIP_eRA,
     "HIP_eDEC" => $HIP_eDEC,
     "HIP_ePlx" => $HIP_ePlx,
     "HIP_epmRA" => $HIP_epmRA,
     "HIP_epmDEC" => $HIP_epmDEC,
     "HIP_BTycho" => $HIP_BTycho,
     "HIP_BTyErr" => $HIP_BTyErr,
     "HIP_VTycho" => $HIP_VTycho,
     "HIP_VTyErr" => $HIP_VmagErr, # ???
     "HIP_BminusV" => $HIP_BminusV,
     "HIP_BmVErr" => $HIP_BmVErr,
     "HIP_VminusI" => $HIP_VminusI,
     "HIP_VmIErr" => $HIP_VmIErr,
     "HIP_Hpmag" => $HIP_Hpmag,
     "HIP_e_Hpmag" => $HIP_e_Hpmag,
     "HIP_Hpscat" => $HIP_Hpscat,
     "HIP_o_Hpmag" => $HIP_o_Hpmag,
     "HIP_DSFlag" => $HIP_DSFlag,
     "HIP_AstSrcFlag" => $HIP_AstSrcFlag,
     "HIP_SolQual" => $HIP_SolQual,
     "HIP_rho" => $HIP_rho,
     "HIP_e_rho" => $HIP_e_rho,
     "HIP_HDnum" => $HIP_HDnum,
     "HIP_DMname" => $HIP_DMname,
     "HIP_DMname2" => $HIP_DMname2,
     "HIP_spType" => $HIP_spType,

     ## New field designations (Note some multiple designations
     ## pointing to the same field
     "HipID" => $HIP_Num,
     "ProxFlag" => $HIP_ProxFlag,
     "raSex" => $HIP_RAhms,
     "decSex" => $HIP_DECdms,
     "Vmag" => $HIP_Vmag,
     "coarseVarFlag" => $HIP_VarFlag,
     "sourceV" => $HIP_sourceV,
     "raDeg" => $HIP_RAdeg,
     "decDeg" => $HIP_DECdeg,
     "parallax" => $HIP_Plx,
     "plx" => $HIP_Plx,
     "pmRA" => $HIP_pmRA,
     "properMotionRA" => $HIP_pmRA,
     "pmDec" => $HIP_pmDEC,
     "properMotionDec" => $HIP_pmDEC,
     "raErr" => $HIP_eRA,
     "decErr" => $HIP_eDEC,
     "plxErr" => $HIP_ePlx,
     "parallaxErr" => $HIP_ePlx,
     "pmRAErr" => $HIP_epmRA,
     "pmDecErr" => $HIP_epmDEC,
     "BTycho" => $HIP_BTycho,
     "BTychoErr" => $HIP_BTyErr,
     "VTycho" => $HIP_VTycho,
     "VTychoErr" => $HIP_VmagErr, # ???
     "BminusV" => $HIP_BminusV,
     "BminusVErr" => $HIP_BmVErr,
     "VminusI" => $HIP_VminusI,
     "VminusIErr" => $HIP_VmIErr,
     "Hpmag" => $HIP_Hpmag,
     "HpmagErr" => $HIP_e_Hpmag,
     "Hpscat" => $HIP_Hpscat,
     "o_Hpmag" => $HIP_o_Hpmag,
     "DSFlag" => $HIP_DSFlag,
     "AstSrcFlag" => $HIP_AstSrcFlag,
     "SolQualFlag" => $HIP_SolQual,
     "PosAng" => $HIP_PosAng,
     "rho" => $HIP_rho,
     "rhoErr" => $HIP_e_rho,
     "deltaH" => $HIP_deltaH,
     "HDnum" => $HIP_HDnum,
     "HDnumber" => $HIP_HDnum,
     "DMname" => $HIP_DMname,
     "DMname2" => $HIP_DMname2,
     "spType" => $HIP_spType,
     "SpType" => $HIP_spType,
     "SpectralType" => $HIP_spType,

     );



#########################################################################
## object constructor -- create hipparcosLine object
#########################################################################
sub new
{

##  declarations  #######################################################

    my $class = shift @_;  ## class designation

    ## Create the hash with the rest of the stuff from the call argument
    ##  list
    my $objRef = { @_ };

## begin  ###############################################################
    if (!(grep m/line/, keys %$objRef))
    {
	die "hipparcos object needs catalog line";
    }

    ## perform the catalog line split
    @{ $objRef->{lineContents} } = split /\|/, $objRef->{line};  ##@lineContents;
    
    ## Establish some fields to help the performance

    # Sometimes the RAdeg field is blank; have to compensate
    if (!($objRef->{raDeg} = trimWhite(${ $objRef->{lineContents} }[$HIP_RAdeg]))) {
	$objRef->{raDeg} = RAdeg(${ $objRef->{lineContents} }[$HIP_RAhms]);
    }

    # Same thing for DECdeg    
    if (!($objRef->{decDeg} = trimWhite(${ $objRef->{lineContents} }[$HIP_DECdeg]))) {
	$objRef->{decDeg} = DECdeg(${ $objRef->{lineContents} }[$HIP_DECdms]);
    }

    $objRef->{BminusV} = trimWhite(${ $objRef->{lineContents} }[$HIP_BminusV]);
    $objRef->{spType} = trimWhite(${ $objRef->{lineContents} }[$HIP_spType]);
    $objRef->{HDnum} = trimWhite(${ $objRef->{lineContents} }[$HIP_HDnum]);
    $objRef->{HIPnum} = trimWhite(${ $objRef->{lineContents} }[$HIP_Num]);

    $objRef->{Vmag} = trimWhite(${ $objRef->{lineContents} }[$HIP_Vmag]);
    $objRef->{parallax} = trimWhite(${ $objRef->{lineContents} }[$HIP_Plx]);
    if (!$objRef->{parallax}) {
	$objRef->{parallax} = 0.0;
	#print STDERR "Defaulting empty parallax to 0 for ", $objRef->{HIPnum}, " (", $objRef->{HDnum}, ")\n";
    }
    ## TLE - 6/2005  HIP parallax in milliarcsec, now put parallax in arcsec
    $objRef->{parallax} = $objRef->{parallax} / 1000.;

    $objRef->{pmRA} = trimWhite(${ $objRef->{lineContents} }[$HIP_pmRA]);
    if (!$objRef->{pmRA}){
	$objRef->{pmRA} = 0.0;
    }
    ## TLE - 6/2005  HIP pmRA in milli-ra-arcsec/yr, now put pmRA in arcsec
    if ($objRef->{decDeg}) {
	$objRef->{pmRA} = $objRef->{pmRA} / 1000. / cos($objRef->{decDeg} * DEG_TO_RAD);
    }

    $objRef->{pmDec} = trimWhite(${ $objRef->{lineContents} }[$HIP_pmDEC]);
    if (!$objRef->{pmDec}) {
	$objRef->{pmDec} = 0.0;
    }
    ## TLE - 6/2005  HIP pmDec in milliarcsec/yr, now put pmDec in arcsec
    $objRef->{pmDec} = $objRef->{pmDec} / 1000.;


    ## anything else?

    ## make, return object
    bless $objRef;
    return $objRef;

} ## end constructor  ###################################################



#########################################################################
## field extraction member function
#########################################################################
sub extractField
{
## declarations  ########################################################
    my $this = shift @_;
    my $field = shift @_;

## begin  ###############################################################
    if (defined $this->{$field})  ## see if there's a field already
    {
	return $this->{$field};
    }
    elsif (grep m/$field/, keys %HipparcosKeys)  ## otherwise dig
    {
	return ${ $this->{lineContents} }[$HipparcosKeys{$field}];
    }
    else ## otherwise return undef
    {
	return undef;
    }

} ## end extractField  ##################################################




#########################################################################
## posJ2000 -- J2000 astrometry
#########################################################################
sub posJ2000
{
## declarations  ########################################################
    my $this = shift @_;
    my $HipTimeOffset = (2000.0 - 1991.25);
    my $masToDeg = MAS_TO_RAD * RAD_TO_DEG;
    my $asToDeg = ASEC_TO_RAD * RAD_TO_DEG;

## begin posJ2000 #######################################################

    ## Hipparcos ideosyncracy -- positions are mean-epoch for the
    ## mission (1991.25), but reported in J2000 (ICRS2000) epoch
    ## orientation.  To compute J2000 positions must project with
    ## proper motion.  Of course, pmRA has a cos(dec) thrown in to
    ## make things interesting

    ## TLE - 6/2005  HIP pmRA was in milli-ra-arcsec/yr, now in arcsec
    ##my $pmRA = $this->{pmRA} / cos($this->{decDeg} * DEG_TO_RAD);

    ##my $raJ2000 = $this->{raDeg}
    ##    + $pmRA * $masToDeg * $HipTimeOffset;
    my $raJ2000 = $this->{raDeg}
        + $this->{pmRA} * $asToDeg * $HipTimeOffset;
	
    ## TLE - 6/2005  HIP pmDec was in milliarcsec/yr, now in arcsec
    ##my $decJ2000 = $this->{decDeg}
    ##    + $this->{pmDec} * $masToDeg * $HipTimeOffset;
    my $decJ2000 = $this->{decDeg}
        + $this->{pmDec} * $asToDeg * $HipTimeOffset;

    return ($raJ2000,$decJ2000);


} ## end posJ2000  ######################################################




return 1;


