package miscArgs;

my $MODVERSION = '$Id: miscArgs.pm,v 1.6 2006/07/21 19:58:12 sgroom Exp $';

# This module contains miscellaneous functions related to the handling
# of program arguments.

use strict;
use warnings;
use Getopt::Long;
use gcConfig;
use findScript;
use miscString;
use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(handleCommonOpts helpFile getArgs);

my $cfg = gcConfig::config();

# Handle the following generic options:
# --copyright, --debug, --help, --longHelp, --version
#
# This function should be called from the main module before that module
# calls GetOptions() itself.
sub handleCommonOpts {
    my $requireArgs = shift;
    my $shortHelpFile = shift;
    my $longHelpFile = shift;

    my $opt_copyright = 0;
    my $opt_debug = 0;
    my $opt_help = 0;
    my $opt_longHelp = 0;
    my $opt_version = 0;
    my $argCount = scalar @ARGV;
    my %opts = ("copyright" => \$opt_copyright,
		"debug+" => \$opt_debug,
		"version" => \$opt_version
		);
    $opts{"help"} = \$opt_help if ($shortHelpFile);
    $opts{"longHelp"} = \$opt_longHelp if ($longHelpFile);
    Getopt::Long::config("pass_through");
    GetOptions(%opts);
    optDebug($opt_debug);	# set the runtime debug flag according to command line opt
    helpFile($opt_copyright, "copyright.txt");
    helpFile($opt_help || ($requireArgs && !$argCount), $shortHelpFile);
    helpFile($opt_longHelp, $longHelpFile);
    if ($opt_version) {
	print $main::MODVERSION . "\n";
	helpFile($opt_version, "version.txt");
    }
}

sub helpFile
{
    my $opt = shift;
    my $f = shift;
    my $want_return = shift;
    return if (!$opt);
    my $cfg = config();
    my $ff = findFile($f, $cfg->gcdocs(), $cfg->gcdata());
    print STDERR "DEBUG: helpFile: file \"$f\" found at \"$ff\"\n" if gcConfig::debug();
    if ($ff) {
	system "cat $ff";
	exit(0) if (!$want_return);
    } else {
	die sprintf("File %s not found in %s or %s", $f, $cfg->gcdocs(),
		    $cfg->gcdata());
    }
}

# Reads a list of arguments from the command line, standard input, or a file.
# Function parameters:
# 0: useStdin (boolean)
# 1: path of a file containing the arguments, one per line;
# alternatively, a reference to an array of filenames
sub getArgs {
    my $useStdin = shift;
    my $fref = shift;
    my @filenames;
    if (@$fref) {
	@filenames = @$fref;
    } else {
	@filenames = ();
    }
    my @args;
    if ($useStdin)
    {
	print STDERR "DEBUG: miscArgs::getArgs: reading from stdin\n" if gcConfig::debug();
	push @args, slurp_lines(<STDIN>);
    } elsif ($#ARGV+1) {
	print STDERR "DEBUG: miscArgs::getArgs: packing words from cmd line\n" if gcConfig::debug();
	foreach my $t (@ARGV) {
	    $t = trimWhite($t);
	    push(@args, $t);
	}
    } elsif (@filenames) {
	print STDERR "DEBUG: miscArgs::getArgs: reading args from files\n" if gcConfig::debug();
	foreach my $filename (@filenames) {
	    open (F, $filename) or die "Unable to open $filename\n";
	    push @args, slurp_lines(<F>);
	}
    }
    return @args;
}

sub slurp_lines {
    my @targets;
    foreach my $line (@_) {
	$line = trimWhite($line);
	if ($line && !($line =~ m/^\#/)) {
	    $line =~ s/\#.*\z//; # Strip trailing comment
	    push(@targets, $line);
	}
    }
    return @targets;
}

return 1;
