package simbadresult;

#
# Data object representing information returned by SIMBAD
#

use strict;
use warnings;

use Exporter();
our @ISA=qw(Exporter);
our @EXPORT=qw(new desig hd hip name description
	    raHMS decDMS raDeg decDeg
	    raHMS1950 decDMS1950 raDeg1950 decDeg1950
	    pmra pmdec
	    spec plx bmag vmag
	    ident notes meas raw);

sub new
{
    my $class = shift @_;
    my $line = shift @_;

    my $ref = { };	# hash representing the result object
    bless $ref;

    if (defined $line)
    {
	$ref->parseSimbadResult($line);
    }

    return $ref;
}

# get/set preferred designation for this object
sub desig
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{desig} = $val;
    }
    return $this->{desig};
}

# get/set HD identifier
sub hd
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{hd} = $val;
    }
    return $this->{hd};
}

# get/set HIP identifier
sub hip
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{hip} = $val;
    }
    return $this->{hip};
}

# get/set NAME identifier
sub name
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{name} = $val;
    }
    return $this->{name};
}

# get/set object description field
sub description
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{description} = $val;
    }
    return $this->{description};
}

# get/set RA as HMS (ICRS J2000.0)
# DOES NOT UPDATE "raDeg"!
sub raHMS
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{raHMS} = $val;
    }
    return $this->{raHMS};
}

# get/set DEC as DMS (ICRS J2000.0)
# DOES NOT UPDATE "decDeg"!
sub decDMS
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{decDMS} = $val;
    }
    return $this->{decDMS};
}

# get/set RA as decimal degrees (ICRS J2000.0)
# DOES NOT UPDATE "raHMS"!
sub raDeg
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{raDeg} = $val;
    }
    return $this->{raDeg};
}

# get/set DEC as decimal degrees (ICRS J2000.0)
# DOES NOT UPDATE "decHMS"!
sub decDeg
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{decDeg} = $val;
    }
    return $this->{decDeg};
}

# get/set RA as HMS (FK4 1950.0/1950.0)
# DOES NOT UPDATE "raDeg"!
sub raHMS1950
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{raHMS1950} = $val;
    }
    return $this->{raHMS1950};
}

# get/set DEC as DMS (FK4 1950.0/1950.0)
# DOES NOT UPDATE "decDeg"!
sub decDMS1950
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{decDMS1950} = $val;
    }
    return $this->{decDMS1950};
}

# get/set RA as decimal degrees (FK4 1950.0/1950.0)
# DOES NOT UPDATE "raHMS"!
sub raDeg1950
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{raDeg1950} = $val;
    }
    return $this->{raDeg1950};
}

# get/set DEC as decimal degrees (FK4 1950.0/1950.0)
# DOES NOT UPDATE "decHMS"!
sub decDeg1950
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{decDeg1950} = $val;
    }
    return $this->{decDeg1950};
}

# get/set proper motion in RA
sub pmra
{
    my $this = shift;
    my $val = shift;
    if ($val)
    {
	$this->{pmra} = $val;
    }
    return $this->{pmra};
}

# get/set proper motion in DEC
sub pmdec
{
    my $this = shift;
    my $val = shift;
    if ($val)
    {
	$this->{pmdec} = $val;
    }
    return $this->{pmdec};
}

# get/set spectral type
sub spec
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{spec} = $val;
    }
    return $this->{spec};
}

# get/set parallax (short form) 
sub plx
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{plx} = $val;
    }
    return $this->{plx};
}

# get/set parallax (SIMBAD long form) 
# parallax with err, qual, and bibcode, in the form
# 65.10 [.76] A 1997A&A...323L..49P
sub plxfull
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{plxfull} = $val;
    }
    return $this->{plxfull};
}

# get/set object B magnitude
sub bmag
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{b} = $val;
    }
    return $this->{b};
}

# get/set object V magnitude
sub vmag
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{v} = $val;
    }
    return $this->{v};
}

# get/set SIMBAD identifiers
# this one takes an array of lines instead of a scalar
sub ident
{
    my $this = shift;
    my @lines = @_;
    if (@lines)
    {
	@{$this->{ident}} = @lines;
    }
    my @val = @{$this->{ident}} if ($this->{ident});
    return @val;
}

# get/set SIMBAD notes
# this one takes an array of lines instead of a scalar
sub notes
{
    my $this = shift;
    my @lines = @_;
    if (@lines)
    {
	@{$this->{notes}} = @lines;
    }
    my @val = @{$this->{notes}} if ($this->{notes});
    return @val;
}

# get/set SIMBAD measurement lines
# this one takes an array of lines instead of a scalar
sub meas
{
    my $this = shift;
    my @lines = @_;
    if (@lines)
    {
	@{$this->{meas}} = @lines;
    }
    my @val = @{$this->{meas}} if ($this->{meas});
    return @val;
}

# get/set raw SIMBAD query results, as one big string
sub raw
{
    my $this = shift;
    my $val = shift;
    if (defined $val)
    {
	$this->{raw} = $val;
    }
    return $this->{raw};
}

1;
