package vector;
my $MODVERSION = '$Id: vector.pm,v 1.4 2008/11/12 23:28:23 sgroom Exp $';
use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(modulus dot norm unit cross);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################


## vector.pm -- 3-vector arithmetic utilities module in Perl



#########################################################################
## modulus -- vector modulus
#########################################################################
sub modulus
{
## begin modulus  #######################################################
    ## Do the sum
    my $result = 0.0;
    for (my $i = 0; $i < 3; $i++)
    {
	my $tmp = shift @_;
	$result += $tmp * $tmp;  ## 
    } # end for loop

    return sqrt($result);

} ## end modulus  #######################################################



#########################################################################
## dot -- dot product
#########################################################################
sub dot
{
## dot (inner) product of two 3-vectors

## declarations dot  ####################################################
    ## extract the call arguments, two 3-vectors
    my (@arg1,@arg2);
    $arg1[0] = shift @_;
    $arg1[1] = shift @_;
    $arg1[2] = shift @_;
    
    $arg2[0] = shift @_;
    $arg2[1] = shift @_;
    $arg2[2] = shift @_;

## begin dot  ###########################################################
    ## Do the dot
    my $result = 0.0;
    for (my $i = 0; $i < 3; $i++)
    {
	$result += $arg1[$i] * $arg2[$i];  ## dot product
    } # end for loop

    return $result;

} ## end dot  ###########################################################


#########################################################################
## norm of a 3-vector
#########################################################################
sub norm
{

    my @arg = @_; ## should only be one list argument

    my $result = 0.0;
    for (my $i = 0; $i < 3; $i++)
    {
	$result += $arg[$i] * $arg[$i];  ## square
    } # end for loop

    return sqrt($result);

} ## end norm  ##########################################################


#########################################################################
## unit vector from a 3-vector
#########################################################################
sub unit
{
    my @arg = @_;

    my $result = 0.0;
    for (my $i = 0; $i < 3; $i++)
    {
	$result += $arg[$i] * $arg[$i];  ## square
    } # end for loop

    $result = sqrt($result);
    for (my $i = 0; $i < 3; $i++)
    {
	$arg[$i] /= $result;
    } # end for loop

    return @arg;


} ## end unit  ##########################################################


#########################################################################
## cross-product of two 3-vectors
#########################################################################
sub cross
{
## declarations cross  ##################################################
    ## extract the call arguments, two 3-vectors
    my (@arg1,@arg2);
    $arg1[0] = shift @_;
    $arg1[1] = shift @_;
    $arg1[2] = shift @_;
    
    $arg2[0] = shift @_;
    $arg2[1] = shift @_;
    $arg2[2] = shift @_;

## begin cross  #########################################################
    ## Explicit computation
    my @result;
    $result[0] = $arg1[1] * $arg2[2]  -  $arg1[2] * $arg2[1];
    $result[1] = $arg1[2] * $arg2[0]  -  $arg1[0] * $arg2[2];
    $result[2] = $arg1[0] * $arg2[1]  -  $arg1[1] * $arg2[0];

    return @result;

} ## end norm  ##########################################################



# end vector
return 1;
