package yearDay;
my $MODVERSION = '$Id: yearDay.pm,v 1.4 2008/11/12 23:28:23 sgroom Exp $';
use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(yearDay);


##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################



##  check if the year "iyear" is a leap year                                  */
##                                                                            */
##  leap years    : 1600, 1984, 1988, 1992, 1996, 2000 etc.    (check = 0)    */
##  no leap years : 1700, 1800, 1900, 2100 etc.                (check = 1)    */

##  data fields : number of days per month                                    */

my @noleapyear = (0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

my @leapyear = (0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

#########################################################################
## ifLeapYear -- leap year check
#########################################################################
sub ifLeapYear
{
    my $year = shift @_;

    if (($year % 100 == 0)
	&& ($year % 400 != 0))
    {
	return 0;
    }
    else
    {
	if ($year % 4 == 0)
	{
	    return 1;
	}
	else
	{
	    return 0;
	}
    }

} ## end ifLeapYear  ####################################################



#########################################################################
## intoMonthDay -- compute month day from yearday
#########################################################################
sub intoMonthDay
{
## declarations intoMonthday  ###########################################
    my $iyear = shift @_;
    my $yearday = shift @_;

## begin intoMonthday  ##################################################
    my $month = 0;
    my $day = 0;
    if (&ifLeapYear($iyear))
      {
	  while ($yearday > 0)
	  {
	      $month++; 
	      $yearday -= $leapyear[$month];
	  }
	  $day = $yearday + $leapyear[$month];
      }
    else
    {
        while ($yearday > 0)
	{
	    $month++;
	    $yearday -= $noleapyear[$month];
	} 
        $day = $yearday + $noleapyear[$month];
      }

} ## end intoMonthDay  ##################################################


#########################################################################
## yearDay -- compute yearday from calendar month day year
#########################################################################
sub yearDay
{
## declarations yearDay  ################################################
    my $year = shift @_;
    my $month = shift @_;
    my $day = shift @_;

    my $yday = $day;

## begin yearDay  #######################################################
    for (my $i = 1; $i < $month; $i++)
    {
	if (&ifLeapYear($year))
	{
	    $yday += $leapyear[$i];
	}
	else
	{
	    $yday += $noleapyear[$i];
	}

    } # end for loop

    return $yday;

} ## end yearDay  #######################################################


1;  ## end yearDay.pm
