#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: simbadQuery.pl,v 1.7 2008/11/12 23:28:40 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules

use Getopt::Long;
use gcConfig;
use gcDebug;
use gcSimbad;
use simbadresult;
use miscArgs;
use astroCatalogue;

my $cfg = gcConfig::config();
my $dbgname = "simbadQuery";	# name of this program for debugging output

my $opt_raw = 0;
my $opt_save = 0;
handleCommonOpts(1,undef,undef);
GetOptions("raw" => \$opt_raw,		# output SIMBAD raw query results only, without formatting
	   "save" => \$opt_save		# save raw query results in file
	);

foreach my $searchname (@ARGV)
{
    # normalize the name and search SIMBAD for it
    my $searchdesig = &normalIdent($searchname);
    my $result = &searchSimbad($searchdesig);
    if (!$result)
    {
	print STDERR "$dbgname: SIMBAD lookup failed for \"$searchdesig\"\n";
    }
    else
    {
	if ($opt_raw)
	{
	    # output the raw results only, without any decoding
	    print ($result->raw() || "");
	}
	else
	{
	    my $desig = ($result->desig() || "");
	    my $hd = ($result->hd() || "");
	    my $hip = ($result->hip() || "");
	    my $name = ($result->name() || "");
	    my $raw = ($result->raw() || "");
	    print "# SIMBAD identified \"$searchdesig\" as \"$desig\" (HD=\"$hd\" HIP=\"$hip\" NAME=\"$name\")\n";

	    my @report = &simbadReport($name,$result,{commonNames=>1,identifiers=>1,meas=>1});
	    print @report;
	}
	if ($opt_save)
	{
	    # Save SIMBAD raw query results in a file.
	    # if GC_SIMBAD_CACHEDIR is set, the file will be written there,
	    # otherwise it will go in the current directory.
	    my $filedesig = &sansSpaces($searchdesig);	# filename will be the search desig but without the spaces
	    my $filename;
	    if($cfg->simbadCacheDir())
	    {
		$filename = $cfg->simbadCacheDir() . '/';
	    }
	    $filename .= $filedesig;

	    # open and write the file
	    if (open CACHEFILE, ">", $filename)
	    {
		print CACHEFILE $result->raw();
		close CACHEFILE;
		print "simbadQuery: wrote raw SIMBAD query results for \"$searchdesig\" to file \"$filename\"\n";
	    }
	    else
	    {
		print STDERR "simbadQuery: could not write to file \"$filename\"\n";
		exit 1;
	    }
	}
    }
}
exit 0;
