#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: skyFormat.pl,v 1.8 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use gcConfig;
use miscArgs;                       # handlecommonOpts

my $cfg = gcConfig::config();
my $GETCAL_RELEASE = gcConfig::version;

### skyFormat -- perl script to convert PTI/KI GUI format to WMKO sky
### format (AB -- 5/2001)

## Usage: skyFormat < in.cat > out.cat
##  or blah blah blah | skyFormat > out.cat

## sample PTI GUI format:
## HDC4676 00 48 58.708 +16 56 26.318 -0.002 -0.202 5.1 3.7 0.4 F8V... 0.0 xxx   xxx  trg

## sample sky format
## HDC4676 00 48 58.708 +16 56 26.318 2000 pmra=-0.002 pmdec=-0.202 5.1 3.7 F8V... 0.0 xxx   xxx  trg

## Parse command-line options
use Getopt::Long;
my $stdin;
my @filenames;
my $echo;
my $allowDups;
handleCommonOpts(); # --version
my $result = GetOptions("stdin" => \$stdin,
		     "filename=s" => \@filenames,
		     "allowdups|allowduplicates|dups|duplicates" => \$allowDups,
		     "echo" => \$echo,
		     "help" => sub {
                 print "Usage: skyFormat [options] skyfile\n";
                 print "\t--stdin\n\t--filename file\n\t--echo\n";
                 print "\t--allowdups | --allowDuplicates | --dups | --duplicates\n";
                 print "\t--help\n";
                 exit 1;
             },
     );


## Slurp the input stream
my @inputStream;
if (($stdin)
    || ((! (scalar @filenames)) && (!(scalar @ARGV)))) ## read from stdin
{
    @inputStream = <STDIN>;
}
elsif (scalar @filenames)  ## otherwise read from designated filenames
{
    foreach my $file (@filenames)
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream , <READ>;
	close (READ);
    } ## end foreach
}
elsif (scalar @ARGV)
{
    foreach my $file (@ARGV)  ## Read all the remaing filenames
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream, <READ>;
	close (READ);
    } ## end foreach
}



my %printed;
foreach my $line (@inputStream)
{
    if (!($line =~ m/^\s*\#/)
	 && ($line =~ m/\S+/))
    {
	## data line -- parse it
	my ($name,$rah,$ram,$ras,$decd,$decm,$decs,$pmra,$pmdec,
	    $V,$K,$BmV,$spType,$rest)
	    = split /\s+/, $line, 14;

	## check if we've printed this target before, if so, skip
	next if !$allowDups && exists($printed{$name});
	$printed{$name} = $name;

	## Mods for sky ideosyncracies -- RLA/AFB
        ## pad the name so that the RA starts in column 17
        $name = sprintf("%-15s",$name);
        ## convert units of pmra to seconds-of-time
        $pmra = sprintf("%-6.4f",$pmra/15);

	## print the sky-formatted output (note hardwire to J2000 coords)
	print "$name $rah $ram $ras $decd $decm $decs 2000 pmra=$pmra pmdec=$pmdec vmag=$V b-v=$BmV $spType $rest";

    }
    elsif ($echo) ## non-data line -- echo it
    {
	print $line;
    }



} ## end while

if ($echo)
{
    print "\n## Processed through skyFormat\n\n";
}

## end skyFormat
