package nullDepth;
my $MODVERSION = '$Id: nullDepth.pm,v 1.2 2008/11/12 23:28:47 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

#########################################################################
## nullDepth.pm
##
## This module contains routines for computing null depth estimates.
## Converted old nullDepth.pl program into a module to be used by timing.
## Steve Groom, October 2007
#########################################################################

use strict;
use warnings;

require 5.000;

use Exporter();

our @ISA = qw(Exporter);
our @EXPORT = qw(nullDepthEstimate);

# getCal modules:
use angles;
use geometry;
use vector;
use baseline;

#########################################################################
## nullDepthEstimate -- return null depth estimates
#########################################################################
sub nullDepthEstimate 
{

## called as: nullDepthEstimate($HA, $angDiam, $targetDec, $NDwavelength, \$bHash{$bl});
# computes null depth estimate for baseline and HA
# returns null depth est, projected baseline length

## declarations nullDepthEstimate  #####################################
    my $HA = shift @_;  ## hour angle (units of degrees)
    my $angDiam = shift @_; ## angular diam (units of mas)
    my $targetDec = shift @_;  ## target declination (units of degrees)
    my $NDwavelength = shift @_; ## wavelength (units of um [microns])
    my $bPtr = shift @_; ## baseline object ptr
    my $baseline = $$bPtr; ## baseline object
    my $blName = $baseline->{name};

## begin nullDepthEstimate  ############################################

    ## compute the null depth at input HA

    my ($dot_bl, $bl_norm, $proj_bl, $nd);
    my @un_vec;


    my @zHat = (0, cos($baseline->{latitude}), sin($baseline->{latitude}));
    $bl_norm = norm( @{$baseline->{baseline} });

    # get unit vector to source?
    @un_vec = unitStar($HA * DEG_TO_RAD,
                       $targetDec * DEG_TO_RAD,
                       $baseline->{latitude} * DEG_TO_RAD);

    # dot product between star vector and baseline
    $dot_bl = dot( @{ $baseline->{baseline} }, @un_vec);
    $dot_bl = abs($dot_bl);

    # projected baseline length
    $proj_bl = sqrt($bl_norm**2 - $dot_bl**2);
  
    ##printf "#    Projected baseline: %8.3f m\n", $proj_bl;
  
    # null depth for that baseline
    $nd = 68972.0 * ($NDwavelength/$angDiam/$proj_bl)**2;
    ##printf "#    Null depth: %10.1f\n", $nd;

    return ($nd, $proj_bl);  ## return our estimates
  
} ## end nullDepthEstimate  ################################################

