package timingMisc;
my $MODVERSION = '$Id: timingMisc.pm,v 1.5 2008/11/12 23:28:47 sgroom Exp $';

use strict;
use warnings;

use Exporter();
our @ISA = qw(Exporter);
our @EXPORT = qw(timeString principalValue stripTiming siderealToSolar solarToSidereal);

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use coordinates;
use novasConstants;

#########################################################################
## timeString -- compose time string from angle (degrees) representation
#########################################################################
sub timeString
{
## declarations timeString  #############################################

    my $val = shift @_;
    my $sigFigs = shift @_;

## begin timeString  ####################################################
    if (!defined $sigFigs)
    {
	$sigFigs = -1;
    }

    my $valPrin = principalValue($val);  ## map value into principal interval (0-360)
    my $timeStr = &RAhms($valPrin,$sigFigs);  ## Render into sexigesimal-like fmt
    $timeStr =~ s/\s+/:/g; ## Use colons rather than spaces

    return $timeStr;

} ## end timeString  ####################################################

#########################################################################
## siderealToSolar -- scale from sidereal time to solar time
#########################################################################
sub siderealToSolar
{
    my $ha = shift;
    return ($ha / novasConstants::SolarToSidereal);
}

#########################################################################
## solarToSidereal -- scale from solar time to sidereal time
#########################################################################
sub solarToSidereal
{
    my $ha = shift;
    return ($ha * novasConstants::SolarToSidereal);
}

#########################################################################
## principalValue -- map angle argument (deg) into principal value range
#########################################################################
sub principalValue
{
    # maps angle in degrees into interval [0,360)
    my $arg = shift @_;

    while ($arg < 0)
    {
	$arg += 360.0;
    }

    while ($arg >= 360.0)
    {
	$arg -= 360.0;
    }

    return $arg;

} ## end principalValue  ################################################

#########################################################################
## stripTiming -- removes timing/calendar annotations from schedules
#########################################################################
sub stripTiming
{
    # This routine replaces the old standalone stripTiming utility.
    # The long regular expression below attempts to match all lines containing
    # annotations that were inserted by the "timing" or "obsCalendar" utilities.
    my $expr =
	"Timing summary|for timings|Using .* Location|Using .* Baseline|The JD at 0 hr UT is|The LST at 0 hr UT is|The Local Time at 0 hr UT is|Approximate UT sunset -- sunrise times|Approximate LST at sunset -- sunrise|Approximate UT moonrise -- moonset times|moon transit|% moon illumination -- |Using .* deg zenith angle constraint|Using .* deg zenith angle .* constraint|Using .* m delay constraint|Using .* delay interval|transits at .*, zenAng|is within zenith angle range from|is never within zenith angle range|is always within zenith angle range|is within delay range from|is always within delay range|is never within delay range|is within pointing range|is never within pointing range|is always within pointing range|is out of pointing range at transit|Observing calendar|Using .* hour transit offsets in accessibilty calculations|is always available in the sky|is never available in the sky|is near transit at .* on|is .* hours .*-transit at .* on";

    # remove all lines from input that match the expression above
    my @out = grep { !/$expr/ } @_;
    return @out;
} ## end stripTiming  ################################################
