#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: xeConvert.pl,v 1.10 2008/11/12 23:28:59 sgroom Exp $';

##########################################################################
# NASA Exoplanet Science Institute (NExScI) -- http://nexsci.caltech.edu #
#       Copyright (c) 1997-2008 California Institute of Technology       #
#                     U.S. Government Sponsorship                        #
# Under contract with the National Aeronautics and Space Administration  #
##########################################################################

use strict;
use warnings;
use FindBin qw($RealBin);
use lib "$RealBin/../lib/modules";		# location of getCal Perl modules
use Getopt::Long;
use gcConfig;                       # sigFigures()
use internalLine;

my $cfg = gcConfig::config();       # sigFigures()

### xeConvert -- convert a PTI GUI catalog into a XEphem-readable format

### use: xeConvert < PTI-file > xephem-file

## MODIFIED T.L. Evans   8 Dec 2004  -- for release 2.7 with new output formats
##   use internalLine class/methods instead of only reading in text
## See comments starting with "TLE"


# keys to the PTI format
# target fk5 1304 = hd 102509 = 93 Leo -- Variable of RS CVn type 
# name      RA         Dec           RApm    Decpm  V    K   B-V  Sptype  extras
# HDC102509 11 47 59.5 +20 13 08.4   0.000   0.000  4.6  4.4 0.46   A7V     0.00  0  +- 0

## Command-line options

Getopt::Long::config("pass_through");
GetOptions("help" => \$help,
           "sigFigures=i" => sub { $cfg->sigFigures($_[1]); },
           "stdin" => \$stdin,
           "filename=s" => \@filenames);


## If the user is looking for help, cat out the help file and exit.
## Short help message
if ($help)
{
    print " usage: xeConvert [options] < GUI catalog > XEphem catalog\n";
    exit(0);
} ## end short help



# Step 2.  Slurp the input
my @inputStream;
if (($stdin)
    || ((! (scalar @filenames)) && (!(scalar @ARGV)))) ## read from stdin
{
    @inputStream = <STDIN>;
}
elsif (scalar @filenames)  ## otherwise read from designated filenames
{
    foreach my $file (@filenames)
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream , <READ>;
	close (READ);
    } ## end foreach
}
elsif (scalar @ARGV)
{
    foreach my $file (@ARGV)  ## Read all the remaing filenames
    {
	open (READ,$file) or die "Unable to open $file\n";
	push @inputStream, <READ>;
	close (READ);
    } ## end foreach
}



## field declarations
$PTIname = 0;
$PTIrah = 1;
$PTIram = 2;
$PTIras = 3;
$PTIdecd = 4;
$PTIdecm = 5;
$PTIdecs = 6;
$PTIvmag = 9;
$PTIkmag = 10;
$PTIBmV = 11;
$PTIsptype = 12;


### Here's the format of an xephem file
#Cha Iota,f|S|F3,9:24:9.1,-80:47:13,5.36,2000

###open(DATAFILE,"93LeoList");

foreach my $line (@inputStream)
{
##  TLE - 12/2004  took out astrometry line matching -- let internalLine class
##        take care of that; here we just ignore any comment or blank input lines

    if (!($line =~ m/^\#/) 
      && ($line =~ m/\S+/)
      && ($line =~ m/^\S+\s+\d+\s+\d+\s+\d+\.*\d*\s+\+*\-*\d+\s+\d+\s+\d+/)) ## astrometry line match

    {
        my $iLine = new internalLine(line => $line);

##  TLE - 12/2004  changed line split, reformat & print into simple 
##                 xephem output method
##  TLE - 6/2005  call correct method name!
##  TLE - 8/2005  added $opt_sigFigures option for decimal -> sexiges conversion for output
        print $iLine->outputXEphemFormat($cfg->sigFigures());

    } ## end non-comment

} ## end while loop 
