#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: testAstroCatalogue.pl,v 1.9 2007/04/23 17:56:13 sgroom Exp $';

#
# testAstroCatalogue - test harness for the astroCatalogue module
#

use strict;
use warnings;
use Test;

BEGIN { plan tests => 83, onfail => sub {die "One or more tests failed"} }

print "Testing astroCatalogue\n";

use astroCatalogue;


print "\t...testing identPieces()\n";
my @p = identPieces('HD 33959');
ok($p[0], 'HD');
ok($p[1], '33959');
ok($p[2], '');

@p = identPieces('HD60179C');
ok($p[0], 'HD');
ok($p[1], '60179C');
ok($p[2], '');

@p = identPieces('GEN# +1.00033959J');
ok($p[0], 'GEN#');
ok($p[1], '+1.00033959J');
ok($p[2], '');

@p = identPieces('AC 211 (=1E 2127+119; M 15)');
ok($p[0], 'AC');
ok($p[1], '211');
ok($p[2], '(=1E 2127+119; M 15)');

@p = identPieces('uvby98 100033959 ABV');
ok($p[0], 'uvby98');
ok($p[1], '100033959');
ok($p[2], 'ABV');

@p = identPieces('ADS 3824 A');
ok($p[0], 'ADS');
ok($p[1], '3824');
ok($p[2], 'A');

@p = identPieces('CCDM J05154+3242A');
ok($p[0], 'CCDM');
ok($p[1], 'J05154+3242A');
ok($p[2], '');

@p = identPieces('TYC 2394- 2028-1');
ok($p[0], 'TYC');
ok($p[1], '2394-');
ok($p[2], '2028-1');

@p = identPieces('IRC+60388');
ok($p[0], 'IRC');
ok($p[1], '+60388');

@p = identPieces('G148-13');
ok($p[0], 'G');
ok($p[1], '148-13');
ok($p[2], '');

@p = identPieces('G 148-13');
ok($p[0], 'G');
ok($p[1], '148-13');
ok($p[2], '');

@p = identPieces('CPD-68 809');
ok($p[0], 'CPD-68');
ok($p[1], '809');
ok($p[2], '');

@p = identPieces('AG+49 864');
ok($p[0], 'AG+49');
ok($p[1], '864');
ok($p[2], '');

@p = identPieces('CSI+50 1633 1');
ok($p[0], 'CSI+50');
ok($p[1], '1633');
ok($p[2], '1');

@p = identPieces('C* 3016');
ok($p[0], 'C*');
ok($p[1], '3016');
ok($p[2], '');

@p = identPieces('CD-31 3520B');
ok($p[0], 'CD-31');
ok($p[1], '3520B');
ok($p[2], '');

@p = identPieces('[SC93b] 267');
ok($p[0], '[SC93b]');
ok($p[1], '267');
ok($p[2], '');

@p = identPieces('[SC93b] I 209');
ok($p[0], '[SC93b] I');
ok($p[1], '209');
ok($p[2], '');

@p = identPieces('[M60b]+36 72');
ok($p[0], '[M60b]');
ok($p[1], '+36 72');
ok($p[2], '');

@p = identPieces('com HD91169=HD109005');
ok($p[0], 'HD');
ok($p[1], '91169');
ok($p[2], '');

@p = identPieces('com HD109005=HD91169');
ok($p[0], 'HD');
ok($p[1], '91169');
ok($p[2], '');

@p = identPieces('p. WDS J2107.4-0814');
ok($p[0], 'p. WDS');
ok($p[1], 'J2107.4-0814');
ok($p[2], '');

@p = identPieces('SKY# 28256');
ok($p[0], 'SKY#');
ok($p[1], '28256');
ok($p[2], '');

@p = identPieces('V* V362 Per');
ok($p[0], 'V*');
ok($p[1], 'V362 Per');
ok($p[2], '');


print "\t...testing stdCatalogue()\n";
ok(stdCatalogue('HDC'), 'HD');
ok(stdCatalogue('ZZZ'), 'ZZZ');

print "\t...testing normalIdent()\n";
ok(normalIdent('HDC187929'), 'HD 187929');
ok(normalIdent('hd187929'), 'HD 187929');
ok(normalIdent('HD60179C'), 'HD 60179C');
ok(normalIdent('HD 60179C'), 'HD 60179C');

print "\t...testing sansSpaces()\n";
ok(sansSpaces('HD60179C'), 'HD_60179C');
ok(sansSpaces('HD60179C',':'), 'HD:60179C');

exit 0;
