#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: testDst.pl,v 1.3 2007/03/06 23:05:41 sgroom Exp $';

#
# testDst - test harness for the dstBounds perl module
# modified to include changes effective in 2007
#

use strict;
use warnings;
use Test;

use dstBounds;
#use Time::JulianDay;

print "Testing dstBounds\n";
BEGIN { plan tests => 65, onfail => sub {die "One or more tests failed"} }

my $y;
my ($jd1, $jd2);
my ($y1, $m1, $d1);
my ($y2, $m2, $d2);

#for (my $y = 1990; $y < 2011; $y++)
#{
#    ($jd1, $jd2) = &dstBounds::dstBounds($y,1);
#    ($y1, $m1, $d1) = inverse_julian_day($jd1);
#    ($y2, $m2, $d2) = inverse_julian_day($jd2);
#    print "y=$y: DST=1: Start jd1=$jd1 ($m1/$d1/$y1), End jd2=$jd2 ($m2/$d2/$y)\n";
#}

# check DST bounds for some years
($jd1, $jd2) = &dstBounds::dstBounds(2004,1);
ok($jd1,2453099.5);	# 2004 US DST begins
ok($jd2,2453309.5);	# 2004 US standard time begins
($jd1, $jd2) = &dstBounds::dstBounds(2005,1);
ok($jd1,2453463.5);	# 2005 US DST begins
ok($jd2,2453673.5);	# 2005 US standard time begins
($jd1, $jd2) = &dstBounds::dstBounds(2006,1);
ok($jd1,2453827.5);	# 2006 US DST begins
ok($jd2,2454037.5);	# 2006 US standard time begins
($jd1, $jd2) = &dstBounds::dstBounds(2007,1);
ok($jd1,2454170.5);	# 2007 US DST begins
ok($jd2,2454408.5);	# 2007 US standard time begins
($jd1, $jd2) = &dstBounds::dstBounds(2008,1);
ok($jd1,2454534.5);	# 2008 US DST begins
ok($jd2,2454772.5);	# 2008 US standard time begins

# check whether DST is in effect on selected dates
ok(!&dstBounds::isDST(2001,3,31,1));	# 3/31/2001 - last day of standard time in US
ok(&dstBounds::isDST(2001,4,1,1));	# 4/1/2001 - start of DST in US
ok(&dstBounds::isDST(2001,10,27,1));	# 10/27/2001 - last day of DST in US
ok(!&dstBounds::isDST(2001,10,28,1));	# 10/28/2001 - start of standard time in US

ok(!&dstBounds::isDST(2002,4,6,1));	# 4/6/2002 - last day of standard time in US
ok(&dstBounds::isDST(2002,4,7,1));	# 4/7/2002 - start of DST in US
ok(&dstBounds::isDST(2002,10,26,1));	# 10/26/2002 - last day of DST in US
ok(!&dstBounds::isDST(2002,10,27,1));	# 10/27/2002 - start of standard time in US

ok(!&dstBounds::isDST(2003,4,5,1));	# 4/5/2003 - last day of standard time in US
ok(&dstBounds::isDST(2003,4,6,1));	# 4/6/2003 - start of DST in US
ok(&dstBounds::isDST(2003,10,25,1));	# 10/25/2003 - last day of DST in US
ok(!&dstBounds::isDST(2003,10,26,1));	# 10/26/2003 - start of standard time in US

ok(!&dstBounds::isDST(2004,4,3,1));	# 4/3/2004 - last day of standard time in US
ok(&dstBounds::isDST(2004,4,4,1));	# 4/4/2004 - start of DST in US
ok(&dstBounds::isDST(2004,10,30,1));	# 10/30/2004 - last day of DST in US
ok(!&dstBounds::isDST(2004,10,31,1));	# 10/31/2004 - start of standard time in US

ok(!&dstBounds::isDST(2005,4,2,1));	# 4/2/2005 - last day of standard time in US
ok(&dstBounds::isDST(2005,4,3,1));	# 4/3/2005 - start of DST in US
ok(&dstBounds::isDST(2005,10,29,1));	# 10/29/2005 - last day of DST in US
ok(!&dstBounds::isDST(2005,10,30,1));	# 10/30/2005 - start of standard time in US

ok(!&dstBounds::isDST(2006,4,1,1));	# 4/1/2006 - last day of standard time in US
ok(&dstBounds::isDST(2006,4,2,1));	# 4/2/2006 - start of DST in US
ok(&dstBounds::isDST(2006,10,28,1));	# 10/28/2006 - last day of DST in US
ok(!&dstBounds::isDST(2006,10,29,1));	# 10/29/2006 - start of standard time in US

ok(!&dstBounds::isDST(2007,1,1,1));	# 1/1/2007 - happy new year
ok(!&dstBounds::isDST(2007,2,14,1));	# 2/14/2007 - happy valentine's day
ok(!&dstBounds::isDST(2007,3,10,1));	# 3/10/2007 - last day of standard time in US
ok(&dstBounds::isDST(2007,3,11,1));	# 3/11/2007 - start of DST in US
ok(&dstBounds::isDST(2007,7,4,1));	# 7/4/2007 - happy fourth of july
ok(&dstBounds::isDST(2007,11,3,1));	# 11/3/2007 - last day of DST in US
ok(!&dstBounds::isDST(2007,11,4,1));	# 11/4/2007 - start of standard time in US

ok(!&dstBounds::isDST(2008,3,8,1));	# 3/8/2008 - last day of standard time in US
ok(&dstBounds::isDST(2008,3,9,1));	# 3/9/2008 - start of DST in US
ok(&dstBounds::isDST(2008,11,1,1));	# 11/1/2008 - last day of DST in US
ok(!&dstBounds::isDST(2008,11,2,1));	# 11/2/2008 - start of standard time in US

ok(!&dstBounds::isDST(2009,3,7,1));	# 3/7/2009 - last day of standard time in US
ok(&dstBounds::isDST(2009,3,8,1));	# 3/8/2009 - start of DST in US
ok(&dstBounds::isDST(2009,10,31,1));	# 10/31/2009 - last day of DST in US
ok(!&dstBounds::isDST(2009,11,1,1));	# 11/1/2009 - start of standard time in US

ok(!&dstBounds::isDST(2010,3,13,1));	# 3/13/2010 - last day of standard time in US
ok(&dstBounds::isDST(2010,3,14,1));	# 3/14/2010 - start of DST in US
ok(&dstBounds::isDST(2010,11,6,1));	# 11/6/2010 - last day of DST in US
ok(!&dstBounds::isDST(2010,11,7,1));	# 11/7/2010 - start of standard time in US

ok(!&dstBounds::isDST(2011,3,12,1));	# 3/12/2011 - last day of standard time in US
ok(&dstBounds::isDST(2011,3,13,1));	# 3/13/2011 - start of DST in US
ok(&dstBounds::isDST(2011,11,5,1));	# 11/5/2011 - last day of DST in US
ok(!&dstBounds::isDST(2011,11,6,1));	# 11/6/2011 - start of standard time in US

ok(!&dstBounds::isDST(2007,1,1,0));	# 1/1/2007 - DST not observed
ok(!&dstBounds::isDST(2007,3,10,0));	# 3/10/2007 - DST not observed
ok(!&dstBounds::isDST(2007,3,11,0));	# 3/11/2007 - DST not observed
ok(!&dstBounds::isDST(2007,7,4,0));	# 7/4/2007 - DST not observed
ok(!&dstBounds::isDST(2007,11,3,0));	# 11/3/2007 - DST not observed
ok(!&dstBounds::isDST(2007,11,4,0));	# 11/4/2007 - DST not observed

ok(!&dstBounds::isDST(2000,2,29,1));	# 2/29/2000 - happy leap day
ok(!&dstBounds::isDST(2004,2,29,1));	# 2/29/2004 - happy leap day
