#! @PERL@
# -*- perl -*-
# @configure_input@

our $MODVERSION = '$Id: testIntegrated.pl,v 1.42 2007/10/25 01:02:35 sgroom Exp $';

#
# testIntegrated - main test script for testing getCal functionality
#

use strict;
use warnings;
use Test;
use miscString;

# create request list 
my @tests;
while (my $test = shift) {
	push @tests, $test;
}

if (@tests) {
	printf "Integrated tests of getCal's components (@tests)\n";
} else {
	printf "Integrated tests of getCal's components (%s version)\n", 
        (want_ext() == 0 ? "standard" : want_ext() == 1 ? "long" : "detailed");
}


# The $STD tests are invoked by 'make check'.  Both $STD and $EXT tests get run
# by 'make check-long'.

# Add new tests in this section as needed
my $STD = 0;    # standard test
my $EXT = 1;    # long test
my $DET = 2;    # detailed


sub want_ext {
    return $ENV{GC_CHECK_EXTENDED} || 0;
}
sub t {
    my $label = shift;
    my $title = shift;
    my $extended = shift;
    my $filter = shift;
    my $cmd = shift;
    # skip test if not extended or not explicitly requested
    return if (($extended && $extended > want_ext() && !@tests) || (@tests && !grep /$label/, @tests));
    my $expected_out_file = sprintf "test%s.expect", $label;
    my $out_file = sprintf "test%s.out", $label;
    my $err_file = sprintf "test%s.err", $label;
    my $tag = "$label: $title ";
    my $status_column = 62;
    print $tag;
    for (my $i = 0; $i < $status_column - length($tag); ++$i) {
	print ".";
    }

    my $exc;
    if ($filter) {
	# filter certain lines from the output before comparing with expected
	$exc = "$cmd 2>$err_file | sed -f exclude.sed > $out_file";
    } else {
	$exc = "$cmd 2>$err_file 1>$out_file";
    }
    system $exc;
    my $diffcmd = "diff $expected_out_file $out_file 2>&1 >/dev/null";
    my $result = system($diffcmd);
    ok($result,0,"Test output differs from expected");

    if (! (-z $err_file) )
    {
	print "Error message file $err_file is non-empty, may indicate a problem with test\n";
    }

    #ok(-z $err_file, 1, "Test produced messages on stderr, set test*.err\n";

    # Throw away the output and stderr if the test was successful
    system "rm -f $out_file" if (!$result);
    system "rm -f $err_file" if (-z $err_file);
}


# How these tests work:
# Each test has a label, a descriptive title, and a command. For a given test--
# say, '02b', for example--the t() function executes the command for that test,
# piping the standard out to test02b.out and the standard error stream to file
# test02b.err.  Then, the contents of test02b.out are compared with file
# test02b.expect (the latter being a CVS file).  If the contents match, the
# test is ruled a success and test02b.out gets deleted.
# If test02b.err is of zero size, is is removed following the test. If it is
# nonzero, is is left in place even if the test was successful.  After running
# 'make check-long', it is a good idea for you to check for any unusual
# messages in *.err.

########################################################################

t('01a', 'no network', $STD, 1,
  'getCal --format=old --targetHD=27901 --noMessages'
  );

t('01b', 'no network, coordinate search', $EXT, 1,
  'getCal --format=old --searchRA=16:28:19 --searchDEC=-24:23:02 --noMessages'
  );

# to validate expected output: diff against 1a
t('01c', 'no network, specified calibrators', $EXT, 1,
  'getCal --format=old --targetHD=27901 --noMessages --cal=HD_24357 --cal=HD25204 --cal="HD 27397" -cal=HD_27946 -cal HD_28556'
  );

t('01d', 'calibrator search, no extra opts', $EXT, 1,
  'getCal --noMessages --format=new --targetHD=187642'
  );

# to validate expected output: diff against 1d
t('01e', 'calibrator search radius', $EXT, 1,
  'getCal --noMessages --format=new --searchRadius=20 --targetHD=187642'
  );

# to validate expected output: diff against 1d
t('01f', 'apparent diameter from effective temperature only', $EXT, 1,
  'getCal --noMessages --format=new --effTemp --targetHD=187642'
  );

# to validate expected output: diff against 1d
t('01g', 'apparent diameter from linear diameter only', $EXT, 1,
  'getCal --noMessages --format=new --linearDiam --targetHD=187642'
  );

# to validate expected output: diff against 1e
t('01h', 'filter by angular diameter', $EXT, 1,
  'getCal --noMessages --format=new --searchRadius=20 --maxAD=0.6 --targetHD=187642'
  );

# to validate expected output: diff against 1e
t('01i', 'filter by luminosity class', $EXT, 1,
  'getCal --noMessages --format=new --searchRadius=20 --lClass=III --targetHD=187642'
  );

# to validate expected output: diff against 1d
t('01j', 'filter by V, K magnitudes', $EXT, 1,
  'getCal --noMessages --format=new --maxV=15 --minV=-1 --maxK=10 --minK=-1 --targetHD=187642'
  );

t('01k', 'filter by V, K and N>6 magnitudes', $EXT, 1,
  'getCal --noMessages --format=new --maxV=15 --minV=-1 --maxK=10 --minK=-1 --targetHD=187642 --minN=6'
  );

t('01l', 'filter by V, K and N<6 magnitudes', $EXT, 1,
  'getCal --noMessages --format=new --maxV=15 --minV=-1 --maxK=10 --minK=-1 --targetHD=187642 --maxN=6'
  );

t('01m', 'no network, coordinate search at 0,0 w/ narrow radius', $EXT, 1,
  'getCal --format=old --searchRA=00:00:00 --searchDEC=-00:00:00 --searchRadius 8. '
  );


########################################################################

t('02a', 'basic invocation, old output format', $STD, 1,
  'gcList --format=old --verbose < names'
  );

# to validate expected output: diff against 2a
t('02b', 'basic invocation, new output format', $EXT, 1,
  'gcList --format=new --verbose < names'
  );

# to validate expected output: diff against 2b
t('02c', 'basic invocation, non-interspersed lines', $EXT, 1,
  'gcList --format=new --noInterspersed < names'
  );

# to validate expected output: diff against 2b
t('02d', 'basic invocation + Nband format', $EXT, 1,
  'gcList --format=new --noMessages --Nband < names'
  );

t('02e', 'basic invocation + Nband + NIRAS', $EXT, 1,
  'gcList --format=new --noMessages --Nband --NIRAS < names'
  );

# to validate expected output: diff against 2b
t('02f', 'basic invocation, new output format, rename targets', $EXT, 1,
  'gcList --format=new --rename --verbose < names'
  );

t('02g', 'basic invocation, old output format, Nband, HDC ', $EXT, 1,
  'gcList --format=old --Nband --verbose --identHDC < names'
  );

t('02h', 'specified calibrators + K2Mass + NIRAS w/diff + noExact', $EXT, 1,
  'GC_TOLERANCE_K=0.01 getCal --format=new --targetHD=27901 --cal=HD_24357 --cal=HD25204 --cal="HD 27397" -cal=HD_27946 -cal HD_28556 -Nband --K2Mass --NIRAS --verbose'
  );

t('02i', 'v1515_cyg no HIP record but values in NIRAS and K2Mass', $EXT, 1,
  'getCal --targetName=v1515_cyg --noCal --K2Mass --NIRAS --verbose'
  );

t('02j', 'basic invocation, new output format, Lband, Lprime', $EXT, 1,
  'gcList --format=new --Lband --Lprime --verbose < names'
  );

t('02k', 'basic invocation, new output format, Lband, Lprime, K2Mass', $EXT, 1,
  'gcList --format=new --Lband --Lprime -K2Mass --verbose < names'
  );



########################################################################

t('03a', 'simbad common names', $STD, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames < names'
  );

# to validate expected output: diff against 3a
t('03b', 'simbad with parallax', $EXT, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames --plx < names'
  );

# to validate expected output: diff against 3a
t('03c', 'simbad with coordinates', $EXT, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames --coord < names'
  );

# to validate expected output: diff against 3c
t('03d', 'simbad with FK5', $EXT, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames --coord --FK5 < names'
  );

# to validate expected output: diff against 3c
t('03e', 'simbad with SAO', $EXT, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames --coord --SAO < names'
  );

# to validate expected output: diff against 3c
t('03f', 'simbad with identifiers', $EXT, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames --coord --identifiers < names'
  );

# to validate expected output: diff against 3c
t('03g', 'simbad with Kmag', $EXT, 1,
  'gcList --format=old --noMessages --simbadInteractiveQuery --commonNames --coord --Kmag < names'
  );

########################################################################

t('04a', 'timing calculations', $STD, 1,
  'gcList --format=old --timing --day=20 --month=9 --year=2000 --location=Palomar --baseline=all --noMessages < names'
  );

# to validate expected output: diff against 4a
t('04b', 'timing calculations, obsCalendar', $EXT, 1,
  'gcList --format=old --timing --day=20 --month=9 --year=2000 --location=Palomar --baseline=all --noMessages --obsCalendar < names'
  );

# to validate expected output: diff against 4a
t('04c', 'timing calculations for Keck', $EXT, 1,
  'gcList --format=old --timing --day=20 --month=9 --year=2000 --location=Keck --baseline=all --noMessages < names'
  );

# to validate expected output: diff against 4a
t('04d', 'timing calculations for VLTI', $EXT, 1,
  'gcList --format=old --timing --day=20 --month=9 --year=2000 --location=VLTI --baseline=all --noMessages < names'
  );

t('04e', 'timing calculations for Keck_Nulling', $EXT, 1,
  'gcList --format=old --timing --day=20 --month=9 --year=2000 --location=Keck_Nulling --baseline=all --noMessages < names'
  );

t('04f', 'timing calculations for CHARA', $EXT, 1,
  'gcList --format=old --timing --day=20 --month=9 --year=2000 --location=CHARA --baseline=all --noMessages < names'
  );

t('04g', 'timing calculations with delay table for PTI', $EXT, 1,
  'gcList --format=new --timing --day=20 --month=9 --year=2000 --location=PTI --baseline=all --table --tableStep 1200 < names'
  );

t('04h', 'timing calculations with nulldepth table for Keck_Nulling', $EXT, 1,
  'gcList --format=new --timing --day=20 --month=9 --year=2000 --location=Keck_Nulling --baseline=all --table --tableStep 1200 --nullDepth --NDstep 1200 < names'
  );

########################################################################

t('05a', 'calibration script composition', $STD, 1,
  'gcList --format=old --calScript --noMessages < names'
  );

# to validate expected output: diff against 5a
t('05b', 'calibration script composition, new format', $EXT, 1,
  'gcList --format=new --calScript --noMessages < names'
  );

# to validate expected output: diff against 5a
t('05c', 'calibration script composition, sky format', $EXT, 1,
  'gcList --format=sky --calScript --noMessages < names'
  );

# to validate expected output: diff against 5a
t('05d', 'calibration script composition, new format, simbad', $EXT, 1,
  'gcList --format=new --calScript --noMessages -simbad< names'
  );


t('05e', 'calibration script composition, named cals', $EXT, 1,
  'getCal --format=new -calScript --targetName ab_aur -cal hd32406 -cal hd186742 -simbad'
  );

########################################################################

t('06a', 'fbol and 2Mass', $STD, 1,
  'gcList --format=old --noMessages --noCal --fbol --noU --strom --geneva --longWL --savePhotometry --2Mass < names'
  );

# to validate expected output: diff against 6a
t('06b', 'fbol, 2Mass, Nband and IRAS', $EXT, 1,
  'gcList --format=old --noMessages --noCal --fbol --noU --strom --geneva --longWL --savePhotometry --2Mass --IRAS --Nband < names'
  );

# to validate expected output: diff against 6a
t('06c', 'fbol, new format', $EXT, 1,
  'gcList --format=new --noMessages --noCal --fbol --noU --strom --geneva --longWL --savePhotometry --2Mass < names'
  );

# to validate expected output: diff against 6c
t('06d', 'fbol with calibrators', $EXT, 1,
  'gcList --format=new --noMessages --fbol --noU --strom --geneva --longWL --savePhotometry --2Mass < names'
  );

# to validate expected output: diff against 6d
t('06e', 'fbol with U-band photometry', $EXT, 1,
  'gcList --format=new --noMessages --fbol --strom --geneva --longWL --savePhotometry --2Mass < names'
  );

# to validate expected output: diff against 6e
t('06f', 'fbol, sans photometry', $EXT, 1,
  'gcList --format=new --noMessages --fbol --strom --geneva --longWL --2Mass < names'
  );

# to validate expected output: diff against 6f
t('06g', 'fbol minimal', $EXT, 1,
  'gcList --format=new --noMessages --fbol -savePhotometry < names'
  );

# to validate expected output: diff against 6g
t('06h', 'fbol constrain temperature', $EXT, 1,
  'gcList --noMessages --format=new --fbol --constrainTemp -savePhotometry < names'
  );

# to validate expected output: diff against 6g
t('06i', 'fbol + CIO', $EXT, 1,
  'gcList --noMessages --format=new --fbol --CIO -savePhotometry < names'
  );

# to validate expected output: diff against 6g
t('06j', 'fbol with fLambda', $EXT, 1,
  'gcList --noMessages --format=new --fbol --fLambda -savePhotometry < names'
  );

# to validate expected output: diff against 6g
t('06k', 'fbol with fNu', $EXT, 1,
  'gcList --noMessages --format=new --fbol --fNu -savePhotometry < names'
  );

# to validate expected output: diff against 6g
t('06l', 'fbol with gezari + hipparcos', $EXT, 1,
  'gcList --noMessages --format=new --fbol --gezari --hipparcos -savePhotometry < names'
  );

# to validate expected output: diff against 6g
t('06m', 'fbol with noB, noPlx, noV', $EXT, 1,
  'gcList --noMessages --format=new --fbol --noB --noPlx --noV -savePhotometry < names'
  );

########################################################################

t('07a', 'basic invocation + Nband + NIRAS + useMeas ', $DET, 0,
  'gcList --format=new --noMessages --Nband --NIRAS --useMeas < names'
  );

t('07b', 'specified calibrators + useMeas + K2Mass w/diff check', $DET, 0,
  'GC_TOLERANCE_K=0.01 getCal --format=new --targetHD=27901 --cal=HD_24357 --cal=HD25204 --cal="HD 27397" -cal=HD_27946 -cal HD_28556 -Nband --K2Mass --verbose --useMeas'
  );

t('07c', 'specified calibrators + useMeas + NIRAS w/diff check', $DET, 0,
  'GC_TOLERANCE_N=0.01 getCal --format=new --targetHD=27901 --cal=HD_24357 --cal=HD25204 --cal="HD 27397" -cal=HD_27946 -cal HD_28556 -Nband --NIRAS --verbose --useMeas'
  );

########################################################################

BEGIN {
    my $count = 6;
    $count += 45 if want_ext() > 0;
    $count += 3 if want_ext() > 1;
    plan tests => ($count),
	onfail => sub {die "One or more of the integrated tests failed";}
}

